# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "AdErrorEnum",
    },
)


class AdErrorEnum(proto.Message):
    r"""Container for enum describing possible ad errors."""

    class AdError(proto.Enum):
        r"""Enum describing possible ad errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            AD_CUSTOMIZERS_NOT_SUPPORTED_FOR_AD_TYPE (2):
                Ad customizers are not supported for ad type.
            APPROXIMATELY_TOO_LONG (3):
                Estimating character sizes the string is too
                long.
            APPROXIMATELY_TOO_SHORT (4):
                Estimating character sizes the string is too
                short.
            BAD_SNIPPET (5):
                There is a problem with the snippet.
            CANNOT_MODIFY_AD (6):
                Cannot modify an ad.
            CANNOT_SET_BUSINESS_NAME_IF_URL_SET (7):
                business name and url cannot be set at the
                same time
            CANNOT_SET_FIELD (8):
                The specified field is incompatible with this
                ad's type or settings.
            CANNOT_SET_FIELD_WITH_ORIGIN_AD_ID_SET (9):
                Cannot set field when originAdId is set.
            CANNOT_SET_FIELD_WITH_AD_ID_SET_FOR_SHARING (10):
                Cannot set field when an existing ad id is
                set for sharing.
            CANNOT_SET_ALLOW_FLEXIBLE_COLOR_FALSE (11):
                Cannot set allowFlexibleColor false if no
                color is provided by user.
            CANNOT_SET_COLOR_CONTROL_WHEN_NATIVE_FORMAT_SETTING (12):
                When user select native, no color control is
                allowed because we will always respect publisher
                color for native format serving.
            CANNOT_SET_URL (13):
                Cannot specify a url for the ad type
            CANNOT_SET_WITHOUT_FINAL_URLS (14):
                Cannot specify a tracking or mobile url
                without also setting final urls
            CANNOT_SET_WITH_FINAL_URLS (15):
                Cannot specify a legacy url and a final url
                simultaneously
            CANNOT_SET_WITH_URL_DATA (17):
                Cannot specify a urls in UrlData and in
                template fields simultaneously.
            CANNOT_USE_AD_SUBCLASS_FOR_OPERATOR (18):
                This operator cannot be used with a subclass
                of Ad.
            CUSTOMER_NOT_APPROVED_MOBILEADS (19):
                Customer is not approved for mobile ads.
            CUSTOMER_NOT_APPROVED_THIRDPARTY_ADS (20):
                Customer is not approved for 3PAS richmedia
                ads.
            CUSTOMER_NOT_APPROVED_THIRDPARTY_REDIRECT_ADS (21):
                Customer is not approved for 3PAS redirect
                richmedia (Ad Exchange) ads.
            CUSTOMER_NOT_ELIGIBLE (22):
                Not an eligible customer
            CUSTOMER_NOT_ELIGIBLE_FOR_UPDATING_BEACON_URL (23):
                Customer is not eligible for updating beacon
                url
            DIMENSION_ALREADY_IN_UNION (24):
                There already exists an ad with the same
                dimensions in the union.
            DIMENSION_MUST_BE_SET (25):
                Ad's dimension must be set before setting
                union dimension.
            DIMENSION_NOT_IN_UNION (26):
                Ad's dimension must be included in the union
                dimensions.
            DISPLAY_URL_CANNOT_BE_SPECIFIED (27):
                Display Url cannot be specified (applies to
                Ad Exchange Ads)
            DOMESTIC_PHONE_NUMBER_FORMAT (28):
                Telephone number contains invalid characters
                or invalid format. Re-enter your number using
                digits (0-9), dashes (-), and parentheses only.
            EMERGENCY_PHONE_NUMBER (29):
                Emergency telephone numbers are not allowed.
                Enter a valid domestic phone number to connect
                customers to your business.
            EMPTY_FIELD (30):
                A required field was not specified or is an
                empty string.
            FEED_ATTRIBUTE_MUST_HAVE_MAPPING_FOR_TYPE_ID (31):
                A feed attribute referenced in an ad
                customizer tag is not in the ad customizer
                mapping for the feed.
            FEED_ATTRIBUTE_MAPPING_TYPE_MISMATCH (32):
                The ad customizer field mapping for the feed
                attribute does not match the expected field
                type.
            ILLEGAL_AD_CUSTOMIZER_TAG_USE (33):
                The use of ad customizer tags in the ad text
                is disallowed. Details in trigger.
            ILLEGAL_TAG_USE (34):
                Tags of the form {PH_x}, where x is a number, are disallowed
                in ad text.
            INCONSISTENT_DIMENSIONS (35):
                The dimensions of the ad are specified or
                derived in multiple ways and are not consistent.
            INCONSISTENT_STATUS_IN_TEMPLATE_UNION (36):
                The status cannot differ among template ads
                of the same union.
            INCORRECT_LENGTH (37):
                The length of the string is not valid.
            INELIGIBLE_FOR_UPGRADE (38):
                The ad is ineligible for upgrade.
            INVALID_AD_ADDRESS_CAMPAIGN_TARGET (39):
                User cannot create mobile ad for countries
                targeted in specified campaign.
            INVALID_AD_TYPE (40):
                Invalid Ad type. A specific type of Ad is
                required.
            INVALID_ATTRIBUTES_FOR_MOBILE_IMAGE (41):
                Headline, description or phone cannot be
                present when creating mobile image ad.
            INVALID_ATTRIBUTES_FOR_MOBILE_TEXT (42):
                Image cannot be present when creating mobile
                text ad.
            INVALID_CALL_TO_ACTION_TEXT (43):
                Invalid call to action text.
            INVALID_CHARACTER_FOR_URL (44):
                Invalid character in URL.
            INVALID_COUNTRY_CODE (45):
                Creative's country code is not valid.
            INVALID_EXPANDED_DYNAMIC_SEARCH_AD_TAG (47):
                Invalid use of Expanded Dynamic Search Ads
                tags ({lpurl} etc.)
            INVALID_INPUT (48):
                An input error whose real reason was not
                properly mapped (should not happen).
            INVALID_MARKUP_LANGUAGE (49):
                An invalid markup language was entered.
            INVALID_MOBILE_CARRIER (50):
                An invalid mobile carrier was entered.
            INVALID_MOBILE_CARRIER_TARGET (51):
                Specified mobile carriers target a country
                not targeted by the campaign.
            INVALID_NUMBER_OF_ELEMENTS (52):
                Wrong number of elements for given element
                type
            INVALID_PHONE_NUMBER_FORMAT (53):
                The format of the telephone number is
                incorrect. Re-enter the number using the correct
                format.
            INVALID_RICH_MEDIA_CERTIFIED_VENDOR_FORMAT_ID (54):
                The certified vendor format id is incorrect.
            INVALID_TEMPLATE_DATA (55):
                The template ad data contains validation
                errors.
            INVALID_TEMPLATE_ELEMENT_FIELD_TYPE (56):
                The template field doesn't have have the
                correct type.
            INVALID_TEMPLATE_ID (57):
                Invalid template id.
            LINE_TOO_WIDE (58):
                After substituting replacement strings, the
                line is too wide.
            MISSING_AD_CUSTOMIZER_MAPPING (59):
                The feed referenced must have ad customizer
                mapping to be used in a customizer tag.
            MISSING_ADDRESS_COMPONENT (60):
                Missing address component in template element
                address field.
            MISSING_ADVERTISEMENT_NAME (61):
                An ad name must be entered.
            MISSING_BUSINESS_NAME (62):
                Business name must be entered.
            MISSING_DESCRIPTION1 (63):
                Description (line 2) must be entered.
            MISSING_DESCRIPTION2 (64):
                Description (line 3) must be entered.
            MISSING_DESTINATION_URL_TAG (65):
                The destination url must contain at least one
                tag (for example, {lpurl})
            MISSING_LANDING_PAGE_URL_TAG (66):
                The tracking url template of
                ExpandedDynamicSearchAd must contain at least
                one tag. (for example, {lpurl})
            MISSING_DIMENSION (67):
                A valid dimension must be specified for this
                ad.
            MISSING_DISPLAY_URL (68):
                A display URL must be entered.
            MISSING_HEADLINE (69):
                Headline must be entered.
            MISSING_HEIGHT (70):
                A height must be entered.
            MISSING_IMAGE (71):
                An image must be entered.
            MISSING_MARKETING_IMAGE_OR_PRODUCT_VIDEOS (72):
                Marketing image or product videos are
                required.
            MISSING_MARKUP_LANGUAGES (73):
                The markup language in which your site is
                written must be entered.
            MISSING_MOBILE_CARRIER (74):
                A mobile carrier must be entered.
            MISSING_PHONE (75):
                Phone number must be entered.
            MISSING_REQUIRED_TEMPLATE_FIELDS (76):
                Missing required template fields
            MISSING_TEMPLATE_FIELD_VALUE (77):
                Missing a required field value
            MISSING_TEXT (78):
                The ad must have text.
            MISSING_VISIBLE_URL (79):
                A visible URL must be entered.
            MISSING_WIDTH (80):
                A width must be entered.
            MULTIPLE_DISTINCT_FEEDS_UNSUPPORTED (81):
                Only 1 feed can be used as the source of ad
                customizer substitutions in a single ad.
            MUST_USE_TEMP_AD_UNION_ID_ON_ADD (82):
                TempAdUnionId must be use when adding
                template ads.
            TOO_LONG (83):
                The string has too many characters.
            TOO_SHORT (84):
                The string has too few characters.
            UNION_DIMENSIONS_CANNOT_CHANGE (85):
                Ad union dimensions cannot change for saved
                ads.
            UNKNOWN_ADDRESS_COMPONENT (86):
                Address component is not {country, lat, lng}.
            UNKNOWN_FIELD_NAME (87):
                Unknown unique field name
            UNKNOWN_UNIQUE_NAME (88):
                Unknown unique name (template element type
                specifier)
            UNSUPPORTED_DIMENSIONS (89):
                Unsupported ad dimension
            URL_INVALID_SCHEME (90):
                URL starts with an invalid scheme.
            URL_INVALID_TOP_LEVEL_DOMAIN (91):
                URL ends with an invalid top-level domain
                name.
            URL_MALFORMED (92):
                URL contains illegal characters.
            URL_NO_HOST (93):
                URL must contain a host name.
            URL_NOT_EQUIVALENT (94):
                URL not equivalent during upgrade.
            URL_HOST_NAME_TOO_LONG (95):
                URL host name too long to be stored as
                visible URL (applies to Ad Exchange ads)
            URL_NO_SCHEME (96):
                URL must start with a scheme.
            URL_NO_TOP_LEVEL_DOMAIN (97):
                URL should end in a valid domain extension,
                such as .com or .net.
            URL_PATH_NOT_ALLOWED (98):
                URL must not end with a path.
            URL_PORT_NOT_ALLOWED (99):
                URL must not specify a port.
            URL_QUERY_NOT_ALLOWED (100):
                URL must not contain a query.
            URL_SCHEME_BEFORE_EXPANDED_DYNAMIC_SEARCH_AD_TAG (102):
                A url scheme is not allowed in front of tag
                in tracking url template (for example,
                http://{lpurl})
            USER_DOES_NOT_HAVE_ACCESS_TO_TEMPLATE (103):
                The user does not have permissions to create
                a template ad for the given template.
            INCONSISTENT_EXPANDABLE_SETTINGS (104):
                Expandable setting is inconsistent/wrong. For
                example, an AdX ad is invalid if it has a
                expandable vendor format but no expanding
                directions specified, or expanding directions is
                specified, but the vendor format is not
                expandable.
            INVALID_FORMAT (105):
                Format is invalid
            INVALID_FIELD_TEXT (106):
                The text of this field did not match a
                pattern of allowed values.
            ELEMENT_NOT_PRESENT (107):
                Template element is mising
            IMAGE_ERROR (108):
                Error occurred during image processing
            VALUE_NOT_IN_RANGE (109):
                The value is not within the valid range
            FIELD_NOT_PRESENT (110):
                Template element field is not present
            ADDRESS_NOT_COMPLETE (111):
                Address is incomplete
            ADDRESS_INVALID (112):
                Invalid address
            VIDEO_RETRIEVAL_ERROR (113):
                Error retrieving specified video
            AUDIO_ERROR (114):
                Error processing audio
            INVALID_YOUTUBE_DISPLAY_URL (115):
                Display URL is incorrect for YouTube PYV ads
            TOO_MANY_PRODUCT_IMAGES (116):
                Too many product Images in GmailAd
            TOO_MANY_PRODUCT_VIDEOS (117):
                Too many product Videos in GmailAd
            INCOMPATIBLE_AD_TYPE_AND_DEVICE_PREFERENCE (118):
                The device preference is not compatible with
                the ad type
            CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY (119):
                Call tracking is not supported for specified
                country.
            CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED (120):
                Carrier specific short number is not allowed.
            DISALLOWED_NUMBER_TYPE (121):
                Specified phone number type is disallowed.
            PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY (122):
                Phone number not supported for country.
            PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY (123):
                Phone number not supported with call tracking
                enabled for country.
            PREMIUM_RATE_NUMBER_NOT_ALLOWED (124):
                Premium rate phone number is not allowed.
            VANITY_PHONE_NUMBER_NOT_ALLOWED (125):
                Vanity phone number is not allowed.
            INVALID_CALL_CONVERSION_TYPE_ID (126):
                Invalid call conversion type id.
            CANNOT_DISABLE_CALL_CONVERSION_AND_SET_CONVERSION_TYPE_ID (127):
                Cannot disable call conversion and set
                conversion type id.
            CANNOT_SET_PATH2_WITHOUT_PATH1 (128):
                Cannot set path2 without path1.
            MISSING_DYNAMIC_SEARCH_ADS_SETTING_DOMAIN_NAME (129):
                Missing domain name in campaign setting when
                adding expanded dynamic search ad.
            INCOMPATIBLE_WITH_RESTRICTION_TYPE (130):
                The associated ad is not compatible with
                restriction type.
            CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED (131):
                Consent for call recording is required for
                creating/updating call only ads. See
                https://support.google.com/google-ads/answer/7412639.
            MISSING_IMAGE_OR_MEDIA_BUNDLE (132):
                Either an image or a media bundle is required
                in a display upload ad.
            PRODUCT_TYPE_NOT_SUPPORTED_IN_THIS_CAMPAIGN (133):
                The display upload product type is not
                supported in this campaign.
            PLACEHOLDER_CANNOT_HAVE_EMPTY_DEFAULT_VALUE (134):
                The default value of an ad placeholder can
                not be the empty string.
            PLACEHOLDER_COUNTDOWN_FUNCTION_CANNOT_HAVE_DEFAULT_VALUE (135):
                Ad placeholders with countdown functions must
                not have a default value.
            PLACEHOLDER_DEFAULT_VALUE_MISSING (136):
                A previous ad placeholder that had a default
                value was found which means that all
                (non-countdown) placeholders must have a default
                value. This ad placeholder does not have a
                default value.
            UNEXPECTED_PLACEHOLDER_DEFAULT_VALUE (137):
                A previous ad placeholder that did not have a
                default value was found which means that no
                placeholders may have a default value. This ad
                placeholder does have a default value.
            AD_CUSTOMIZERS_MAY_NOT_BE_ADJACENT (138):
                Two ad customizers may not be directly
                adjacent in an ad text. They must be separated
                by at least one character.
            UPDATING_AD_WITH_NO_ENABLED_ASSOCIATION (139):
                The ad is not associated with any enabled
                AdGroupAd, and cannot be updated.
            CALL_AD_VERIFICATION_URL_FINAL_URL_DOES_NOT_HAVE_SAME_DOMAIN (140):
                Call Ad verification url and final url don't
                have same domain.
            CALL_AD_FINAL_URL_AND_VERIFICATION_URL_CANNOT_BOTH_BE_EMPTY (154):
                Final url and verification url cannot both be
                empty for call ads.
            TOO_MANY_AD_CUSTOMIZERS (141):
                Too many ad customizers in one asset.
            INVALID_AD_CUSTOMIZER_FORMAT (142):
                The ad customizer tag is recognized, but the
                format is invalid.
            NESTED_AD_CUSTOMIZER_SYNTAX (143):
                Customizer tags cannot be nested.
            UNSUPPORTED_AD_CUSTOMIZER_SYNTAX (144):
                The ad customizer syntax used in the ad is
                not supported.
            UNPAIRED_BRACE_IN_AD_CUSTOMIZER_TAG (145):
                There exists unpaired brace in the ad
                customizer tag.
            MORE_THAN_ONE_COUNTDOWN_TAG_TYPE_EXISTS (146):
                More than one type of countdown tag exists
                among all text lines.
            DATE_TIME_IN_COUNTDOWN_TAG_IS_INVALID (147):
                Date time in the countdown tag is invalid.
            DATE_TIME_IN_COUNTDOWN_TAG_IS_PAST (148):
                Date time in the countdown tag is in the
                past.
            UNRECOGNIZED_AD_CUSTOMIZER_TAG_FOUND (149):
                Cannot recognize the ad customizer tag.
            CUSTOMIZER_TYPE_FORBIDDEN_FOR_FIELD (150):
                Customizer type forbidden for this field.
            INVALID_CUSTOMIZER_ATTRIBUTE_NAME (151):
                Customizer attribute name is invalid.
            STORE_MISMATCH (152):
                App store value does not match the value of
                the app store in the app specified in the
                campaign.
            MISSING_REQUIRED_IMAGE_ASPECT_RATIO (153):
                Missing required image aspect ratio.
            MISMATCHED_ASPECT_RATIOS (155):
                Aspect ratios mismatch between different
                assets.
            DUPLICATE_IMAGE_ACROSS_CAROUSEL_CARDS (156):
                Images must be unique between different
                carousel card assets.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        AD_CUSTOMIZERS_NOT_SUPPORTED_FOR_AD_TYPE = 2
        APPROXIMATELY_TOO_LONG = 3
        APPROXIMATELY_TOO_SHORT = 4
        BAD_SNIPPET = 5
        CANNOT_MODIFY_AD = 6
        CANNOT_SET_BUSINESS_NAME_IF_URL_SET = 7
        CANNOT_SET_FIELD = 8
        CANNOT_SET_FIELD_WITH_ORIGIN_AD_ID_SET = 9
        CANNOT_SET_FIELD_WITH_AD_ID_SET_FOR_SHARING = 10
        CANNOT_SET_ALLOW_FLEXIBLE_COLOR_FALSE = 11
        CANNOT_SET_COLOR_CONTROL_WHEN_NATIVE_FORMAT_SETTING = 12
        CANNOT_SET_URL = 13
        CANNOT_SET_WITHOUT_FINAL_URLS = 14
        CANNOT_SET_WITH_FINAL_URLS = 15
        CANNOT_SET_WITH_URL_DATA = 17
        CANNOT_USE_AD_SUBCLASS_FOR_OPERATOR = 18
        CUSTOMER_NOT_APPROVED_MOBILEADS = 19
        CUSTOMER_NOT_APPROVED_THIRDPARTY_ADS = 20
        CUSTOMER_NOT_APPROVED_THIRDPARTY_REDIRECT_ADS = 21
        CUSTOMER_NOT_ELIGIBLE = 22
        CUSTOMER_NOT_ELIGIBLE_FOR_UPDATING_BEACON_URL = 23
        DIMENSION_ALREADY_IN_UNION = 24
        DIMENSION_MUST_BE_SET = 25
        DIMENSION_NOT_IN_UNION = 26
        DISPLAY_URL_CANNOT_BE_SPECIFIED = 27
        DOMESTIC_PHONE_NUMBER_FORMAT = 28
        EMERGENCY_PHONE_NUMBER = 29
        EMPTY_FIELD = 30
        FEED_ATTRIBUTE_MUST_HAVE_MAPPING_FOR_TYPE_ID = 31
        FEED_ATTRIBUTE_MAPPING_TYPE_MISMATCH = 32
        ILLEGAL_AD_CUSTOMIZER_TAG_USE = 33
        ILLEGAL_TAG_USE = 34
        INCONSISTENT_DIMENSIONS = 35
        INCONSISTENT_STATUS_IN_TEMPLATE_UNION = 36
        INCORRECT_LENGTH = 37
        INELIGIBLE_FOR_UPGRADE = 38
        INVALID_AD_ADDRESS_CAMPAIGN_TARGET = 39
        INVALID_AD_TYPE = 40
        INVALID_ATTRIBUTES_FOR_MOBILE_IMAGE = 41
        INVALID_ATTRIBUTES_FOR_MOBILE_TEXT = 42
        INVALID_CALL_TO_ACTION_TEXT = 43
        INVALID_CHARACTER_FOR_URL = 44
        INVALID_COUNTRY_CODE = 45
        INVALID_EXPANDED_DYNAMIC_SEARCH_AD_TAG = 47
        INVALID_INPUT = 48
        INVALID_MARKUP_LANGUAGE = 49
        INVALID_MOBILE_CARRIER = 50
        INVALID_MOBILE_CARRIER_TARGET = 51
        INVALID_NUMBER_OF_ELEMENTS = 52
        INVALID_PHONE_NUMBER_FORMAT = 53
        INVALID_RICH_MEDIA_CERTIFIED_VENDOR_FORMAT_ID = 54
        INVALID_TEMPLATE_DATA = 55
        INVALID_TEMPLATE_ELEMENT_FIELD_TYPE = 56
        INVALID_TEMPLATE_ID = 57
        LINE_TOO_WIDE = 58
        MISSING_AD_CUSTOMIZER_MAPPING = 59
        MISSING_ADDRESS_COMPONENT = 60
        MISSING_ADVERTISEMENT_NAME = 61
        MISSING_BUSINESS_NAME = 62
        MISSING_DESCRIPTION1 = 63
        MISSING_DESCRIPTION2 = 64
        MISSING_DESTINATION_URL_TAG = 65
        MISSING_LANDING_PAGE_URL_TAG = 66
        MISSING_DIMENSION = 67
        MISSING_DISPLAY_URL = 68
        MISSING_HEADLINE = 69
        MISSING_HEIGHT = 70
        MISSING_IMAGE = 71
        MISSING_MARKETING_IMAGE_OR_PRODUCT_VIDEOS = 72
        MISSING_MARKUP_LANGUAGES = 73
        MISSING_MOBILE_CARRIER = 74
        MISSING_PHONE = 75
        MISSING_REQUIRED_TEMPLATE_FIELDS = 76
        MISSING_TEMPLATE_FIELD_VALUE = 77
        MISSING_TEXT = 78
        MISSING_VISIBLE_URL = 79
        MISSING_WIDTH = 80
        MULTIPLE_DISTINCT_FEEDS_UNSUPPORTED = 81
        MUST_USE_TEMP_AD_UNION_ID_ON_ADD = 82
        TOO_LONG = 83
        TOO_SHORT = 84
        UNION_DIMENSIONS_CANNOT_CHANGE = 85
        UNKNOWN_ADDRESS_COMPONENT = 86
        UNKNOWN_FIELD_NAME = 87
        UNKNOWN_UNIQUE_NAME = 88
        UNSUPPORTED_DIMENSIONS = 89
        URL_INVALID_SCHEME = 90
        URL_INVALID_TOP_LEVEL_DOMAIN = 91
        URL_MALFORMED = 92
        URL_NO_HOST = 93
        URL_NOT_EQUIVALENT = 94
        URL_HOST_NAME_TOO_LONG = 95
        URL_NO_SCHEME = 96
        URL_NO_TOP_LEVEL_DOMAIN = 97
        URL_PATH_NOT_ALLOWED = 98
        URL_PORT_NOT_ALLOWED = 99
        URL_QUERY_NOT_ALLOWED = 100
        URL_SCHEME_BEFORE_EXPANDED_DYNAMIC_SEARCH_AD_TAG = 102
        USER_DOES_NOT_HAVE_ACCESS_TO_TEMPLATE = 103
        INCONSISTENT_EXPANDABLE_SETTINGS = 104
        INVALID_FORMAT = 105
        INVALID_FIELD_TEXT = 106
        ELEMENT_NOT_PRESENT = 107
        IMAGE_ERROR = 108
        VALUE_NOT_IN_RANGE = 109
        FIELD_NOT_PRESENT = 110
        ADDRESS_NOT_COMPLETE = 111
        ADDRESS_INVALID = 112
        VIDEO_RETRIEVAL_ERROR = 113
        AUDIO_ERROR = 114
        INVALID_YOUTUBE_DISPLAY_URL = 115
        TOO_MANY_PRODUCT_IMAGES = 116
        TOO_MANY_PRODUCT_VIDEOS = 117
        INCOMPATIBLE_AD_TYPE_AND_DEVICE_PREFERENCE = 118
        CALLTRACKING_NOT_SUPPORTED_FOR_COUNTRY = 119
        CARRIER_SPECIFIC_SHORT_NUMBER_NOT_ALLOWED = 120
        DISALLOWED_NUMBER_TYPE = 121
        PHONE_NUMBER_NOT_SUPPORTED_FOR_COUNTRY = 122
        PHONE_NUMBER_NOT_SUPPORTED_WITH_CALLTRACKING_FOR_COUNTRY = 123
        PREMIUM_RATE_NUMBER_NOT_ALLOWED = 124
        VANITY_PHONE_NUMBER_NOT_ALLOWED = 125
        INVALID_CALL_CONVERSION_TYPE_ID = 126
        CANNOT_DISABLE_CALL_CONVERSION_AND_SET_CONVERSION_TYPE_ID = 127
        CANNOT_SET_PATH2_WITHOUT_PATH1 = 128
        MISSING_DYNAMIC_SEARCH_ADS_SETTING_DOMAIN_NAME = 129
        INCOMPATIBLE_WITH_RESTRICTION_TYPE = 130
        CUSTOMER_CONSENT_FOR_CALL_RECORDING_REQUIRED = 131
        MISSING_IMAGE_OR_MEDIA_BUNDLE = 132
        PRODUCT_TYPE_NOT_SUPPORTED_IN_THIS_CAMPAIGN = 133
        PLACEHOLDER_CANNOT_HAVE_EMPTY_DEFAULT_VALUE = 134
        PLACEHOLDER_COUNTDOWN_FUNCTION_CANNOT_HAVE_DEFAULT_VALUE = 135
        PLACEHOLDER_DEFAULT_VALUE_MISSING = 136
        UNEXPECTED_PLACEHOLDER_DEFAULT_VALUE = 137
        AD_CUSTOMIZERS_MAY_NOT_BE_ADJACENT = 138
        UPDATING_AD_WITH_NO_ENABLED_ASSOCIATION = 139
        CALL_AD_VERIFICATION_URL_FINAL_URL_DOES_NOT_HAVE_SAME_DOMAIN = 140
        CALL_AD_FINAL_URL_AND_VERIFICATION_URL_CANNOT_BOTH_BE_EMPTY = 154
        TOO_MANY_AD_CUSTOMIZERS = 141
        INVALID_AD_CUSTOMIZER_FORMAT = 142
        NESTED_AD_CUSTOMIZER_SYNTAX = 143
        UNSUPPORTED_AD_CUSTOMIZER_SYNTAX = 144
        UNPAIRED_BRACE_IN_AD_CUSTOMIZER_TAG = 145
        MORE_THAN_ONE_COUNTDOWN_TAG_TYPE_EXISTS = 146
        DATE_TIME_IN_COUNTDOWN_TAG_IS_INVALID = 147
        DATE_TIME_IN_COUNTDOWN_TAG_IS_PAST = 148
        UNRECOGNIZED_AD_CUSTOMIZER_TAG_FOUND = 149
        CUSTOMIZER_TYPE_FORBIDDEN_FOR_FIELD = 150
        INVALID_CUSTOMIZER_ATTRIBUTE_NAME = 151
        STORE_MISMATCH = 152
        MISSING_REQUIRED_IMAGE_ASPECT_RATIO = 153
        MISMATCHED_ASPECT_RATIOS = 155
        DUPLICATE_IMAGE_ACROSS_CAROUSEL_CARDS = 156


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.errors",
    marshal="google.ads.googleads.v20",
    manifest={
        "OperationAccessDeniedErrorEnum",
    },
)


class OperationAccessDeniedErrorEnum(proto.Message):
    r"""Container for enum describing possible operation access
    denied errors.

    """

    class OperationAccessDeniedError(proto.Enum):
        r"""Enum describing possible operation access denied errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            ACTION_NOT_PERMITTED (2):
                Unauthorized invocation of a service's method
                (get, mutate, etc.)
            CREATE_OPERATION_NOT_PERMITTED (3):
                Unauthorized CREATE operation in invoking a
                service's mutate method.
            REMOVE_OPERATION_NOT_PERMITTED (4):
                Unauthorized REMOVE operation in invoking a
                service's mutate method.
            UPDATE_OPERATION_NOT_PERMITTED (5):
                Unauthorized UPDATE operation in invoking a
                service's mutate method.
            MUTATE_ACTION_NOT_PERMITTED_FOR_CLIENT (6):
                A mutate action is not allowed on this
                resource, from this client.
            OPERATION_NOT_PERMITTED_FOR_CAMPAIGN_TYPE (7):
                This operation is not permitted on this
                campaign type
            CREATE_AS_REMOVED_NOT_PERMITTED (8):
                A CREATE operation may not set status to
                REMOVED.
            OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE (9):
                This operation is not allowed because the
                resource is removed.
            OPERATION_NOT_PERMITTED_FOR_AD_GROUP_TYPE (10):
                This operation is not permitted on this ad
                group type.
            MUTATE_NOT_PERMITTED_FOR_CUSTOMER (11):
                The mutate is not allowed for this customer.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        ACTION_NOT_PERMITTED = 2
        CREATE_OPERATION_NOT_PERMITTED = 3
        REMOVE_OPERATION_NOT_PERMITTED = 4
        UPDATE_OPERATION_NOT_PERMITTED = 5
        MUTATE_ACTION_NOT_PERMITTED_FOR_CLIENT = 6
        OPERATION_NOT_PERMITTED_FOR_CAMPAIGN_TYPE = 7
        CREATE_AS_REMOVED_NOT_PERMITTED = 8
        OPERATION_NOT_PERMITTED_FOR_REMOVED_RESOURCE = 9
        OPERATION_NOT_PERMITTED_FOR_AD_GROUP_TYPE = 10
        MUTATE_NOT_PERMITTED_FOR_CUSTOMER = 11


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "ConversionActionErrorEnum",
    },
)


class ConversionActionErrorEnum(proto.Message):
    r"""Container for enum describing possible conversion action
    errors.

    """

    class ConversionActionError(proto.Enum):
        r"""Enum describing possible conversion action errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            DUPLICATE_NAME (2):
                The specified conversion action name already
                exists.
            DUPLICATE_APP_ID (3):
                Another conversion action with the specified
                app id already exists.
            TWO_CONVERSION_ACTIONS_BIDDING_ON_SAME_APP_DOWNLOAD (4):
                Android first open action conflicts with
                Google play codeless download action tracking
                the same app.
            BIDDING_ON_SAME_APP_DOWNLOAD_AS_GLOBAL_ACTION (5):
                Android first open action conflicts with
                Google play codeless download action tracking
                the same app.
            DATA_DRIVEN_MODEL_WAS_NEVER_GENERATED (6):
                The attribution model cannot be set to DATA_DRIVEN because a
                data-driven model has never been generated.
            DATA_DRIVEN_MODEL_EXPIRED (7):
                The attribution model cannot be set to DATA_DRIVEN because
                the data-driven model is expired.
            DATA_DRIVEN_MODEL_STALE (8):
                The attribution model cannot be set to DATA_DRIVEN because
                the data-driven model is stale.
            DATA_DRIVEN_MODEL_UNKNOWN (9):
                The attribution model cannot be set to DATA_DRIVEN because
                the data-driven model is unavailable or the conversion
                action was newly added.
            CREATION_NOT_SUPPORTED (10):
                Creation of this conversion action type isn't
                supported by Google Ads API.
            UPDATE_NOT_SUPPORTED (11):
                Update of this conversion action isn't
                supported by Google Ads API.
            CANNOT_SET_RULE_BASED_ATTRIBUTION_MODELS (12):
                Rule-based attribution models are deprecated
                and not allowed to be set by conversion action.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DUPLICATE_NAME = 2
        DUPLICATE_APP_ID = 3
        TWO_CONVERSION_ACTIONS_BIDDING_ON_SAME_APP_DOWNLOAD = 4
        BIDDING_ON_SAME_APP_DOWNLOAD_AS_GLOBAL_ACTION = 5
        DATA_DRIVEN_MODEL_WAS_NEVER_GENERATED = 6
        DATA_DRIVEN_MODEL_EXPIRED = 7
        DATA_DRIVEN_MODEL_STALE = 8
        DATA_DRIVEN_MODEL_UNKNOWN = 9
        CREATION_NOT_SUPPORTED = 10
        UPDATE_NOT_SUPPORTED = 11
        CANNOT_SET_RULE_BASED_ATTRIBUTION_MODELS = 12


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import (
    offline_conversion_diagnostic_status_enum,
)
from google.ads.googleads.v22.enums.types import (
    offline_event_upload_client_enum,
)
from google.ads.googleads.v22.resources.types import (
    offline_conversion_upload_client_summary,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "OfflineConversionUploadConversionActionSummary",
    },
)


class OfflineConversionUploadConversionActionSummary(proto.Message):
    r"""Offline conversion upload summary at conversion action level.

    Attributes:
        resource_name (str):
            Output only. The resource name of the offline conversion
            upload summary at conversion action level. Offline
            conversion upload conversion action summary resource names
            have the form:

            ``customers/{customer_id}/offlineConversionUploadConversionActionSummaries/{conversion_action_id}~{client}``
        client (google.ads.googleads.v22.enums.types.OfflineEventUploadClientEnum.OfflineEventUploadClient):
            Output only. Client type of the upload event.
        conversion_action_id (int):
            Output only. Conversion action id.
        conversion_action_name (str):
            Output only. The name of the conversion
            action.
        status (google.ads.googleads.v22.enums.types.OfflineConversionDiagnosticStatusEnum.OfflineConversionDiagnosticStatus):
            Output only. Overall status for offline
            conversion upload conversion action summary.
            Status is generated from most recent calendar
            day with upload stats.
        total_event_count (int):
            Output only. Total count of uploaded events.
        successful_event_count (int):
            Output only. Total count of successful
            uploaded events.
        pending_event_count (int):
            Output only. Total count of pending uploaded
            events.
        last_upload_date_time (str):
            Output only. Date for the latest upload
            batch. The format is "yyyy-mm-dd hh:mm:ss", and
            it's in the time zone of the Google Ads account.
        daily_summaries (MutableSequence[google.ads.googleads.v22.resources.types.OfflineConversionSummary]):
            Output only. Summary of history stats by last
            N days.
        job_summaries (MutableSequence[google.ads.googleads.v22.resources.types.OfflineConversionSummary]):
            Output only. Summary of history stats by last
            N jobs.
        alerts (MutableSequence[google.ads.googleads.v22.resources.types.OfflineConversionAlert]):
            Output only. Details for each error code.
            Alerts are generated from most recent calendar
            day with upload stats.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    client: (
        offline_event_upload_client_enum.OfflineEventUploadClientEnum.OfflineEventUploadClient
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=offline_event_upload_client_enum.OfflineEventUploadClientEnum.OfflineEventUploadClient,
    )
    conversion_action_id: int = proto.Field(
        proto.INT64,
        number=3,
    )
    conversion_action_name: str = proto.Field(
        proto.STRING,
        number=4,
    )
    status: (
        offline_conversion_diagnostic_status_enum.OfflineConversionDiagnosticStatusEnum.OfflineConversionDiagnosticStatus
    ) = proto.Field(
        proto.ENUM,
        number=5,
        enum=offline_conversion_diagnostic_status_enum.OfflineConversionDiagnosticStatusEnum.OfflineConversionDiagnosticStatus,
    )
    total_event_count: int = proto.Field(
        proto.INT64,
        number=6,
    )
    successful_event_count: int = proto.Field(
        proto.INT64,
        number=7,
    )
    pending_event_count: int = proto.Field(
        proto.INT64,
        number=8,
    )
    last_upload_date_time: str = proto.Field(
        proto.STRING,
        number=9,
    )
    daily_summaries: MutableSequence[
        offline_conversion_upload_client_summary.OfflineConversionSummary
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message=offline_conversion_upload_client_summary.OfflineConversionSummary,
    )
    job_summaries: MutableSequence[
        offline_conversion_upload_client_summary.OfflineConversionSummary
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=11,
        message=offline_conversion_upload_client_summary.OfflineConversionSummary,
    )
    alerts: MutableSequence[
        offline_conversion_upload_client_summary.OfflineConversionAlert
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=12,
        message=offline_conversion_upload_client_summary.OfflineConversionAlert,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "LocalServicesVerificationStatusEnum",
    },
)


class LocalServicesVerificationStatusEnum(proto.Message):
    r"""Container for enum describing status of a particular Local
    Services Ads verification category.

    """

    class LocalServicesVerificationStatus(proto.Enum):
        r"""Enum describing status of a particular Local Services Ads
        verification category.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Unknown verification status.
            NEEDS_REVIEW (2):
                Verification has started, but has not
                finished.
            FAILED (3):
                Verification has failed.
            PASSED (4):
                Verification has passed.
            NOT_APPLICABLE (5):
                Verification is not applicable.
            NO_SUBMISSION (6):
                Verification is required but pending
                submission.
            PARTIAL_SUBMISSION (7):
                Not all required verification has been
                submitted.
            PENDING_ESCALATION (8):
                Verification needs review by Local Services
                Ads Ops Specialist.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        NEEDS_REVIEW = 2
        FAILED = 3
        PASSED = 4
        NOT_APPLICABLE = 5
        NO_SUBMISSION = 6
        PARTIAL_SUBMISSION = 7
        PENDING_ESCALATION = 8


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v22.common.types import campaign_goal_settings
from google.ads.googleads.v22.enums.types import goal_type as gage_goal_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "CampaignGoalConfig",
    },
)


class CampaignGoalConfig(proto.Message):
    r"""A link between a campaign and a goal enabling
    campaign-specific optimization.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. The resource name of the campaign goal config.
            campaign goal config resource names have the form:
            ``customers/{customer_id}/campaignGoalConfigs/{campaign_id}~{goal_id}``
        campaign (str):
            Immutable. The resource name of the campaign
            for this link.
        goal (str):
            Immutable. The resource name of the goal this
            link is attached to.
        goal_type (google.ads.googleads.v22.enums.types.GoalTypeEnum.GoalType):
            Output only. The goal type this link is
            attached to.
        campaign_retention_settings (google.ads.googleads.v22.common.types.CampaignGoalSettings.CampaignRetentionGoalSettings):
            Retention goal campaign settings.

            This field is a member of `oneof`_ ``campaign_goal_config_settings``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=2,
    )
    goal: str = proto.Field(
        proto.STRING,
        number=3,
    )
    goal_type: gage_goal_type.GoalTypeEnum.GoalType = proto.Field(
        proto.ENUM,
        number=4,
        enum=gage_goal_type.GoalTypeEnum.GoalType,
    )
    campaign_retention_settings: (
        campaign_goal_settings.CampaignGoalSettings.CampaignRetentionGoalSettings
    ) = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="campaign_goal_config_settings",
        message=campaign_goal_settings.CampaignGoalSettings.CampaignRetentionGoalSettings,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v19.enums",
    marshal="google.ads.googleads.v19",
    manifest={
        "ConversionActionCategoryEnum",
    },
)


class ConversionActionCategoryEnum(proto.Message):
    r"""Container for enum describing the category of conversions
    that are associated with a ConversionAction.

    """

    class ConversionActionCategory(proto.Enum):
        r"""The category of conversions that are associated with a
        ConversionAction.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            DEFAULT (2):
                Default category.
            PAGE_VIEW (3):
                User visiting a page.
            PURCHASE (4):
                Purchase, sales, or "order placed" event.
            SIGNUP (5):
                Signup user action.
            DOWNLOAD (7):
                Software download action (as for an app).
            ADD_TO_CART (8):
                The addition of items to a shopping cart or
                bag on an advertiser site.
            BEGIN_CHECKOUT (9):
                When someone enters the checkout flow on an
                advertiser site.
            SUBSCRIBE_PAID (10):
                The start of a paid subscription for a
                product or service.
            PHONE_CALL_LEAD (11):
                A call to indicate interest in an
                advertiser's offering.
            IMPORTED_LEAD (12):
                A lead conversion imported from an external
                source into Google Ads.
            SUBMIT_LEAD_FORM (13):
                A submission of a form on an advertiser site
                indicating business interest.
            BOOK_APPOINTMENT (14):
                A booking of an appointment with an
                advertiser's business.
            REQUEST_QUOTE (15):
                A quote or price estimate request.
            GET_DIRECTIONS (16):
                A search for an advertiser's business
                location with intention to visit.
            OUTBOUND_CLICK (17):
                A click to an advertiser's partner's site.
            CONTACT (18):
                A call, SMS, email, chat or other type of
                contact to an advertiser.
            ENGAGEMENT (19):
                A website engagement event such as long site
                time or a Google Analytics (GA) Smart Goal.
                Intended to be used for GA, Firebase, GA Gold
                goal imports.
            STORE_VISIT (20):
                A visit to a physical store location.
            STORE_SALE (21):
                A sale occurring in a physical store.
            QUALIFIED_LEAD (22):
                A lead conversion imported from an external
                source into Google Ads, that has been further
                qualified by the advertiser (marketing/sales
                team). In the lead-to-sale journey, advertisers
                get leads, then act on them by reaching out to
                the consumer. If the consumer is interested and
                may end up buying their product, the advertiser
                marks such leads as "qualified leads".
            CONVERTED_LEAD (23):
                A lead conversion imported from an external
                source into Google Ads, that has further
                completed a chosen stage as defined by the lead
                gen advertiser.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DEFAULT = 2
        PAGE_VIEW = 3
        PURCHASE = 4
        SIGNUP = 5
        DOWNLOAD = 7
        ADD_TO_CART = 8
        BEGIN_CHECKOUT = 9
        SUBSCRIBE_PAID = 10
        PHONE_CALL_LEAD = 11
        IMPORTED_LEAD = 12
        SUBMIT_LEAD_FORM = 13
        BOOK_APPOINTMENT = 14
        REQUEST_QUOTE = 15
        GET_DIRECTIONS = 16
        OUTBOUND_CLICK = 17
        CONTACT = 18
        ENGAGEMENT = 19
        STORE_VISIT = 20
        STORE_SALE = 21
        QUALIFIED_LEAD = 22
        CONVERTED_LEAD = 23


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.enums",
    marshal="google.ads.googleads.v21",
    manifest={
        "LinkedProductTypeEnum",
    },
)


class LinkedProductTypeEnum(proto.Message):
    r"""Container for enum describing different types of linked
    products.

    """

    class LinkedProductType(proto.Enum):
        r"""Describes the possible link types for a link between a Google
        Ads customer and another product.

        Values:
            UNSPECIFIED (0):
                Not specified.
            UNKNOWN (1):
                Used for return value only. Represents value
                unknown in this version.
            DATA_PARTNER (2):
                A link to Data partner.
            GOOGLE_ADS (3):
                A link to Google Ads.
            HOTEL_CENTER (7):
                A link to Hotel Center.
            MERCHANT_CENTER (8):
                A link to Google Merchant Center.
            ADVERTISING_PARTNER (9):
                A link to the Google Ads account of the
                advertising partner.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DATA_PARTNER = 2
        GOOGLE_ADS = 3
        HOTEL_CENTER = 7
        MERCHANT_CENTER = 8
        ADVERTISING_PARTNER = 9


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "CustomerLabel",
    },
)


class CustomerLabel(proto.Message):
    r"""Represents a relationship between a customer and a label.
    This customer may not have access to all the labels attached to
    it. Additional CustomerLabels may be returned by increasing
    permissions with login-customer-id.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Immutable. Name of the resource. Customer label resource
            names have the form:
            ``customers/{customer_id}/customerLabels/{label_id}``
        customer (str):
            Output only. The resource name of the
            customer to which the label is attached. Read
            only.

            This field is a member of `oneof`_ ``_customer``.
        label (str):
            Output only. The resource name of the label
            assigned to the customer.
            Note: the Customer ID portion of the label
            resource name is not validated when creating a
            new CustomerLabel.

            This field is a member of `oneof`_ ``_label``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    customer: str = proto.Field(
        proto.STRING,
        number=4,
        optional=True,
    )
    label: str = proto.Field(
        proto.STRING,
        number=5,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.enums.types import (
    asset_field_type as gage_asset_field_type,
)
from google.ads.googleads.v21.enums.types import call_to_action_type
from google.ads.googleads.v21.enums.types import hotel_asset_suggestion_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "SuggestTravelAssetsRequest",
        "SuggestTravelAssetsResponse",
        "HotelAssetSuggestion",
        "HotelTextAsset",
        "HotelImageAsset",
    },
)


class SuggestTravelAssetsRequest(proto.Message):
    r"""Request message for
    [TravelAssetSuggestionService.SuggestTravelAssets][google.ads.googleads.v21.services.TravelAssetSuggestionService.SuggestTravelAssets].

    Attributes:
        customer_id (str):
            Required. The ID of the customer.
        language_option (str):
            Required. The language specifications in BCP
            47 format (for example, en-US, zh-CN, etc.) for
            the asset suggestions. Text will be in this
            language. Usually matches one of the campaign
            target languages.
        place_ids (MutableSequence[str]):
            The Google Maps Place IDs of hotels for which
            assets are requested. See
            https://developers.google.com/places/web-service/place-id
            for more information.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    language_option: str = proto.Field(
        proto.STRING,
        number=2,
    )
    place_ids: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )


class SuggestTravelAssetsResponse(proto.Message):
    r"""Response message for
    [TravelAssetSuggestionService.SuggestTravelAssets][google.ads.googleads.v21.services.TravelAssetSuggestionService.SuggestTravelAssets].

    Attributes:
        hotel_asset_suggestions (MutableSequence[google.ads.googleads.v21.services.types.HotelAssetSuggestion]):
            Asset suggestions for each place ID submitted
            in the request.
    """

    hotel_asset_suggestions: MutableSequence["HotelAssetSuggestion"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="HotelAssetSuggestion",
        )
    )


class HotelAssetSuggestion(proto.Message):
    r"""Message containing the asset suggestions for a hotel.

    Attributes:
        place_id (str):
            Google Places ID of the hotel.
        final_url (str):
            Suggested final URL for an AssetGroup.
        hotel_name (str):
            Hotel name in requested language.
        call_to_action (google.ads.googleads.v21.enums.types.CallToActionTypeEnum.CallToActionType):
            Call to action type.
        text_assets (MutableSequence[google.ads.googleads.v21.services.types.HotelTextAsset]):
            Text assets such as headline, description,
            etc.
        image_assets (MutableSequence[google.ads.googleads.v21.services.types.HotelImageAsset]):
            Image assets such as
            landscape/portrait/square, etc.
        status (google.ads.googleads.v21.enums.types.HotelAssetSuggestionStatusEnum.HotelAssetSuggestionStatus):
            The status of the hotel asset suggestion.
    """

    place_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    final_url: str = proto.Field(
        proto.STRING,
        number=2,
    )
    hotel_name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    call_to_action: (
        call_to_action_type.CallToActionTypeEnum.CallToActionType
    ) = proto.Field(
        proto.ENUM,
        number=4,
        enum=call_to_action_type.CallToActionTypeEnum.CallToActionType,
    )
    text_assets: MutableSequence["HotelTextAsset"] = proto.RepeatedField(
        proto.MESSAGE,
        number=5,
        message="HotelTextAsset",
    )
    image_assets: MutableSequence["HotelImageAsset"] = proto.RepeatedField(
        proto.MESSAGE,
        number=6,
        message="HotelImageAsset",
    )
    status: (
        hotel_asset_suggestion_status.HotelAssetSuggestionStatusEnum.HotelAssetSuggestionStatus
    ) = proto.Field(
        proto.ENUM,
        number=7,
        enum=hotel_asset_suggestion_status.HotelAssetSuggestionStatusEnum.HotelAssetSuggestionStatus,
    )


class HotelTextAsset(proto.Message):
    r"""A single text asset suggestion for a hotel.

    Attributes:
        text (str):
            Asset text in requested language.
        asset_field_type (google.ads.googleads.v21.enums.types.AssetFieldTypeEnum.AssetFieldType):
            The text asset type. For example, HEADLINE,
            DESCRIPTION, etc.
    """

    text: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_field_type: (
        gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )


class HotelImageAsset(proto.Message):
    r"""A single image asset suggestion for a hotel.

    Attributes:
        uri (str):
            URI for the image.
        asset_field_type (google.ads.googleads.v21.enums.types.AssetFieldTypeEnum.AssetFieldType):
            The Image asset type. For example, MARKETING_IMAGE,
            PORTRAIT_MARKETING_IMAGE, etc.
    """

    uri: str = proto.Field(
        proto.STRING,
        number=1,
    )
    asset_field_type: (
        gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType
    ) = proto.Field(
        proto.ENUM,
        number=2,
        enum=gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "ExperimentArm",
    },
)


class ExperimentArm(proto.Message):
    r"""A Google ads experiment for users to experiment changes on
    multiple campaigns, compare the performance, and apply the
    effective changes.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the experiment arm.
            Experiment arm resource names have the form:

            ``customers/{customer_id}/experimentArms/{TrialArm.trial_id}~{TrialArm.trial_arm_id}``
        experiment (str):
            Immutable. The experiment to which the
            ExperimentArm belongs.
        name (str):
            Required. The name of the experiment arm. It
            must have a minimum length of 1 and maximum
            length of 1024. It must be unique under an
            experiment.
        control (bool):
            Whether this arm is a control arm. A control
            arm is the arm against which the other arms are
            compared.
        traffic_split (int):
            Traffic split of the trial arm. The value
            should be between 1 and 100 and must total 100
            between the two trial arms.
        campaigns (MutableSequence[str]):
            List of campaigns in the trial arm. The max
            length is one.
        in_design_campaigns (MutableSequence[str]):
            Output only. The in design campaigns in the
            treatment experiment arm.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    experiment: str = proto.Field(
        proto.STRING,
        number=8,
    )
    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    control: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    traffic_split: int = proto.Field(
        proto.INT64,
        number=5,
    )
    campaigns: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=6,
    )
    in_design_campaigns: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=7,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.common.types import consent as gagc_consent
from google.ads.googleads.v22.enums.types import (
    user_identifier_source as gage_user_identifier_source,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.common",
    marshal="google.ads.googleads.v22",
    manifest={
        "OfflineUserAddressInfo",
        "UserIdentifier",
        "TransactionAttribute",
        "StoreAttribute",
        "ItemAttribute",
        "UserData",
        "UserAttribute",
        "EventAttribute",
        "EventItemAttribute",
        "ShoppingLoyalty",
        "CustomerMatchUserListMetadata",
        "StoreSalesMetadata",
        "StoreSalesThirdPartyMetadata",
    },
)


class OfflineUserAddressInfo(proto.Message):
    r"""Address identifier of offline data.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        hashed_first_name (str):
            First name of the user, which is hashed as
            SHA-256 after normalized (Lowercase all
            characters; Remove any extra spaces before,
            after, and in between).

            This field is a member of `oneof`_ ``_hashed_first_name``.
        hashed_last_name (str):
            Last name of the user, which is hashed as
            SHA-256 after normalized (lower case only and no
            punctuation).

            This field is a member of `oneof`_ ``_hashed_last_name``.
        city (str):
            City of the address. Only accepted for Store
            Sales and ConversionAdjustmentUploadService.

            This field is a member of `oneof`_ ``_city``.
        state (str):
            State code of the address. Only accepted for
            Store Sales and
            ConversionAdjustmentUploadService.

            This field is a member of `oneof`_ ``_state``.
        country_code (str):
            2-letter country code in ISO-3166-1 alpha-2
            of the user's address.

            This field is a member of `oneof`_ ``_country_code``.
        postal_code (str):
            Postal code of the user's address.

            This field is a member of `oneof`_ ``_postal_code``.
        hashed_street_address (str):
            The street address of the user hashed using
            SHA-256 hash function after normalization (lower
            case only). Only accepted for
            ConversionAdjustmentUploadService.

            This field is a member of `oneof`_ ``_hashed_street_address``.
    """

    hashed_first_name: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    hashed_last_name: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    city: str = proto.Field(
        proto.STRING,
        number=9,
        optional=True,
    )
    state: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    country_code: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    postal_code: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    hashed_street_address: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )


class UserIdentifier(proto.Message):
    r"""User identifying information.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        user_identifier_source (google.ads.googleads.v22.enums.types.UserIdentifierSourceEnum.UserIdentifierSource):
            Source of the user identifier when the upload
            is from Store Sales, ConversionUploadService, or
            ConversionAdjustmentUploadService.
        hashed_email (str):
            Hashed email address using SHA-256 hash
            function after normalization. Accepted for
            Customer Match, Store Sales,
            ConversionUploadService, and
            ConversionAdjustmentUploadService.

            This field is a member of `oneof`_ ``identifier``.
        hashed_phone_number (str):
            Hashed phone number using SHA-256 hash
            function after normalization (E164 standard).
            Accepted for Customer Match, Store Sales,
            ConversionUploadService, and
            ConversionAdjustmentUploadService.

            This field is a member of `oneof`_ ``identifier``.
        mobile_id (str):
            Mobile device ID (advertising ID/IDFA).
            Accepted only for Customer Match.

            This field is a member of `oneof`_ ``identifier``.
        third_party_user_id (str):
            Advertiser-assigned user ID for Customer
            Match upload, or third-party-assigned user ID
            for Store Sales. Accepted only for Customer
            Match and Store Sales.

            This field is a member of `oneof`_ ``identifier``.
        address_info (google.ads.googleads.v22.common.types.OfflineUserAddressInfo):
            Address information. Accepted only for
            Customer Match, Store Sales, and
            ConversionAdjustmentUploadService.

            This field is a member of `oneof`_ ``identifier``.
    """

    user_identifier_source: (
        gage_user_identifier_source.UserIdentifierSourceEnum.UserIdentifierSource
    ) = proto.Field(
        proto.ENUM,
        number=6,
        enum=gage_user_identifier_source.UserIdentifierSourceEnum.UserIdentifierSource,
    )
    hashed_email: str = proto.Field(
        proto.STRING,
        number=7,
        oneof="identifier",
    )
    hashed_phone_number: str = proto.Field(
        proto.STRING,
        number=8,
        oneof="identifier",
    )
    mobile_id: str = proto.Field(
        proto.STRING,
        number=9,
        oneof="identifier",
    )
    third_party_user_id: str = proto.Field(
        proto.STRING,
        number=10,
        oneof="identifier",
    )
    address_info: "OfflineUserAddressInfo" = proto.Field(
        proto.MESSAGE,
        number=5,
        oneof="identifier",
        message="OfflineUserAddressInfo",
    )


class TransactionAttribute(proto.Message):
    r"""Attribute of the store sales transaction.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        transaction_date_time (str):
            Timestamp when transaction occurred. Required. The format is
            "YYYY-MM-DD HH:MM:SS[+/-HH:MM]", where [+/-HH:MM] is an
            optional timezone offset from UTC. If the offset is absent,
            the API will use the account's timezone as default.
            Examples: "2018-03-05 09:15:00" or "2018-02-01
            14:34:30+03:00".

            This field is a member of `oneof`_ ``_transaction_date_time``.
        transaction_amount_micros (float):
            Transaction amount in micros. Required.
            Transaction amount in micros needs to be greater
            than 1000. If item Attributes are provided, it
            represents the total value of the items, after
            multiplying the unit price per item by the
            quantity provided in the ItemAttributes.

            This field is a member of `oneof`_ ``_transaction_amount_micros``.
        currency_code (str):
            Transaction currency code. ISO 4217
            three-letter code is used. Required.

            This field is a member of `oneof`_ ``_currency_code``.
        conversion_action (str):
            The resource name of conversion action to
            report conversions to. Required.

            This field is a member of `oneof`_ ``_conversion_action``.
        order_id (str):
            Transaction order id. Useful to group
            transactions which are part of the same order.

            This field is a member of `oneof`_ ``_order_id``.
        store_attribute (google.ads.googleads.v22.common.types.StoreAttribute):
            Store attributes of the transaction.
        custom_value (str):
            Value of the custom variable for each
            transaction. Allowed only if a custom key is
            provided in the store sales metadata.

            This field is a member of `oneof`_ ``_custom_value``.
        item_attribute (google.ads.googleads.v22.common.types.ItemAttribute):
            Item attributes of the transaction.
            Accessible only to customers on the allow-list.
    """

    transaction_date_time: str = proto.Field(
        proto.STRING,
        number=8,
        optional=True,
    )
    transaction_amount_micros: float = proto.Field(
        proto.DOUBLE,
        number=9,
        optional=True,
    )
    currency_code: str = proto.Field(
        proto.STRING,
        number=10,
        optional=True,
    )
    conversion_action: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    order_id: str = proto.Field(
        proto.STRING,
        number=12,
        optional=True,
    )
    store_attribute: "StoreAttribute" = proto.Field(
        proto.MESSAGE,
        number=6,
        message="StoreAttribute",
    )
    custom_value: str = proto.Field(
        proto.STRING,
        number=13,
        optional=True,
    )
    item_attribute: "ItemAttribute" = proto.Field(
        proto.MESSAGE,
        number=14,
        message="ItemAttribute",
    )


class StoreAttribute(proto.Message):
    r"""Store attributes of the transaction.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        store_code (str):
            Store code from
            https://support.google.com/business/answer/3370250#storecode

            This field is a member of `oneof`_ ``_store_code``.
    """

    store_code: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )


class ItemAttribute(proto.Message):
    r"""Item attributes of the transaction.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        item_id (str):
            A unique identifier of a product. It must be
            the exact same Merchant Center Item ID you use
            in your Google Merchant Center for this product.
            Required.
        merchant_id (int):
            ID of the Merchant Center Account.
            Required.

            This field is a member of `oneof`_ ``_merchant_id``.
        country_code (str):
            Common Locale Data Repository (CLDR) territory code of the
            country associated with the feed where your items are
            uploaded. See
            https://developers.google.com/google-ads/api/reference/data/codes-formats#country-codes
            for more information. This information is useful to
            differentiate product information in cases where a product
            (identified by item_id) is associated with multiple
            countries.
        language_code (str):
            ISO 639-1 code of the language associated with the feed
            where your items are uploaded. This information is useful to
            differentiate product information in cases where a product
            (identified by item_id) is associated with multiple
            languages.
        quantity (int):
            The number of items sold. Defaults to 1 if
            not set.
    """

    item_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    merchant_id: int = proto.Field(
        proto.INT64,
        number=2,
        optional=True,
    )
    country_code: str = proto.Field(
        proto.STRING,
        number=3,
    )
    language_code: str = proto.Field(
        proto.STRING,
        number=4,
    )
    quantity: int = proto.Field(
        proto.INT64,
        number=5,
    )


class UserData(proto.Message):
    r"""User data holding user identifiers and attributes.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        user_identifiers (MutableSequence[google.ads.googleads.v22.common.types.UserIdentifier]):
            User identification info. Required.
        transaction_attribute (google.ads.googleads.v22.common.types.TransactionAttribute):
            Additional transactions/attributes associated
            with the user. Required when updating store
            sales data.
        user_attribute (google.ads.googleads.v22.common.types.UserAttribute):
            Additional attributes associated with the
            user. Required when updating customer match
            attributes. These have an expiration of 540
            days.
        consent (google.ads.googleads.v22.common.types.Consent):
            The consent setting for the user. If set,
            will override the job level consent for this
            user.

            This field is a member of `oneof`_ ``_consent``.
    """

    user_identifiers: MutableSequence["UserIdentifier"] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message="UserIdentifier",
    )
    transaction_attribute: "TransactionAttribute" = proto.Field(
        proto.MESSAGE,
        number=2,
        message="TransactionAttribute",
    )
    user_attribute: "UserAttribute" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="UserAttribute",
    )
    consent: gagc_consent.Consent = proto.Field(
        proto.MESSAGE,
        number=4,
        optional=True,
        message=gagc_consent.Consent,
    )


class UserAttribute(proto.Message):
    r"""User attribute, can only be used with CUSTOMER_MATCH_WITH_ATTRIBUTES
    job type.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        lifetime_value_micros (int):
            Advertiser defined lifetime value for the
            user.

            This field is a member of `oneof`_ ``_lifetime_value_micros``.
        lifetime_value_bucket (int):
            Advertiser defined lifetime value bucket for
            the user. The valid range for a lifetime value
            bucket is from 1 (low) to 10 (high), except for
            remove operation where 0 will also be accepted.

            This field is a member of `oneof`_ ``_lifetime_value_bucket``.
        last_purchase_date_time (str):
            Timestamp of the last purchase made by the user. The format
            is YYYY-MM-DD HH:MM:SS[+/-HH:MM], where [+/-HH:MM] is an
            optional timezone offset from UTC. If the offset is absent,
            the API will use the account's timezone as default.
        average_purchase_count (int):
            Advertiser defined average number of
            purchases that are made by the user in a 30 day
            period.
        average_purchase_value_micros (int):
            Advertiser defined average purchase value in
            micros for the user.
        acquisition_date_time (str):
            Timestamp when the user was acquired. The format is
            YYYY-MM-DD HH:MM:SS[+/-HH:MM], where [+/-HH:MM] is an
            optional timezone offset from UTC. If the offset is absent,
            the API will use the account's timezone as default.
        shopping_loyalty (google.ads.googleads.v22.common.types.ShoppingLoyalty):
            The shopping loyalty related data. Shopping
            utilizes this data to provide users with a
            better experience. Accessible only to merchants
            on the allow-list with the user's consent.

            This field is a member of `oneof`_ ``_shopping_loyalty``.
        lifecycle_stage (str):
            Optional. Advertiser defined lifecycle stage
            for the user. The accepted values are "Lead",
            "Active" and "Churned".
        first_purchase_date_time (str):
            Optional. Timestamp of the first purchase made by the user.
            The format is YYYY-MM-DD HH:MM:SS[+/-HH:MM], where
            [+/-HH:MM] is an optional timezone offset from UTC. If the
            offset is absent, the API will use the account's timezone as
            default.
        event_attribute (MutableSequence[google.ads.googleads.v22.common.types.EventAttribute]):
            Optional. Advertiser defined events and their
            attributes. All the values in the nested fields
            are required. Currently this field is in beta.
    """

    lifetime_value_micros: int = proto.Field(
        proto.INT64,
        number=1,
        optional=True,
    )
    lifetime_value_bucket: int = proto.Field(
        proto.INT32,
        number=2,
        optional=True,
    )
    last_purchase_date_time: str = proto.Field(
        proto.STRING,
        number=3,
    )
    average_purchase_count: int = proto.Field(
        proto.INT32,
        number=4,
    )
    average_purchase_value_micros: int = proto.Field(
        proto.INT64,
        number=5,
    )
    acquisition_date_time: str = proto.Field(
        proto.STRING,
        number=6,
    )
    shopping_loyalty: "ShoppingLoyalty" = proto.Field(
        proto.MESSAGE,
        number=7,
        optional=True,
        message="ShoppingLoyalty",
    )
    lifecycle_stage: str = proto.Field(
        proto.STRING,
        number=8,
    )
    first_purchase_date_time: str = proto.Field(
        proto.STRING,
        number=9,
    )
    event_attribute: MutableSequence["EventAttribute"] = proto.RepeatedField(
        proto.MESSAGE,
        number=10,
        message="EventAttribute",
    )


class EventAttribute(proto.Message):
    r"""Advertiser defined events and their attributes. All the
    values in the nested fields are required.

    Attributes:
        event (str):
            Required. Advertiser defined event to be used
            for remarketing. The accepted values are
            "Viewed", "Cart", "Purchased" and "Recommended".
        event_date_time (str):
            Required. Timestamp at which the event happened. The format
            is YYYY-MM-DD HH:MM:SS[+/-HH:MM], where [+/-HH:MM] is an
            optional timezone offset from UTC. If the offset is absent,
            the API will use the account's timezone as default.
        item_attribute (MutableSequence[google.ads.googleads.v22.common.types.EventItemAttribute]):
            Required. Item attributes of the event.
    """

    event: str = proto.Field(
        proto.STRING,
        number=1,
    )
    event_date_time: str = proto.Field(
        proto.STRING,
        number=2,
    )
    item_attribute: MutableSequence["EventItemAttribute"] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message="EventItemAttribute",
    )


class EventItemAttribute(proto.Message):
    r"""Event Item attributes of the Customer Match.

    Attributes:
        item_id (str):
            Optional. A unique identifier of a product.
            It can be either the Merchant Center Item ID or
            GTIN (Global Trade Item Number).
    """

    item_id: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ShoppingLoyalty(proto.Message):
    r"""The shopping loyalty related data. Shopping utilizes this
    data to provide users with a better experience.
    Accessible only to merchants on the allow-list.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        loyalty_tier (str):
            The membership tier. It is a free-form string
            as each merchant may have their own loyalty
            system. For example, it could be a number from 1
            to 10, or a string such as "Golden" or "Silver",
            or even empty string "".

            This field is a member of `oneof`_ ``_loyalty_tier``.
    """

    loyalty_tier: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )


class CustomerMatchUserListMetadata(proto.Message):
    r"""Metadata for customer match user list.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        user_list (str):
            The resource name of remarketing list to update data.
            Required for job of CUSTOMER_MATCH_USER_LIST type.

            This field is a member of `oneof`_ ``_user_list``.
        consent (google.ads.googleads.v22.common.types.Consent):
            The consent setting for all the users in this
            job.

            This field is a member of `oneof`_ ``_consent``.
    """

    user_list: str = proto.Field(
        proto.STRING,
        number=2,
        optional=True,
    )
    consent: gagc_consent.Consent = proto.Field(
        proto.MESSAGE,
        number=3,
        optional=True,
        message=gagc_consent.Consent,
    )


class StoreSalesMetadata(proto.Message):
    r"""Metadata for Store Sales Direct.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        loyalty_fraction (float):
            This is the fraction of all transactions that
            are identifiable (for example, associated with
            any form of customer information). Required. The
            fraction needs to be between 0 and 1 (excluding
            0).

            This field is a member of `oneof`_ ``_loyalty_fraction``.
        transaction_upload_fraction (float):
            This is the ratio of sales being uploaded
            compared to the overall sales that can be
            associated with a customer. Required. The
            fraction needs to be between 0 and 1 (excluding
            0). For example, if you upload half the sales
            that you are able to associate with a customer,
            this would be 0.5.

            This field is a member of `oneof`_ ``_transaction_upload_fraction``.
        custom_key (str):
            Name of the store sales custom variable key.
            A predefined key that can be applied to the
            transaction and then later used for custom
            segmentation in reporting.
            Accessible only to customers on the allow-list.

            This field is a member of `oneof`_ ``_custom_key``.
        third_party_metadata (google.ads.googleads.v22.common.types.StoreSalesThirdPartyMetadata):
            Metadata for a third party Store Sales
            upload.
    """

    loyalty_fraction: float = proto.Field(
        proto.DOUBLE,
        number=5,
        optional=True,
    )
    transaction_upload_fraction: float = proto.Field(
        proto.DOUBLE,
        number=6,
        optional=True,
    )
    custom_key: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    third_party_metadata: "StoreSalesThirdPartyMetadata" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="StoreSalesThirdPartyMetadata",
    )


class StoreSalesThirdPartyMetadata(proto.Message):
    r"""Metadata for a third party Store Sales.
    This product is only for customers on the allow-list. Contact
    your Google business development representative for details on
    the upload configuration.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        advertiser_upload_date_time (str):
            Time the advertiser uploaded the data to the
            partner. Required. The format is "YYYY-MM-DD
            HH:MM:SS". Examples: "2018-03-05 09:15:00" or
            "2018-02-01 14:34:30".

            This field is a member of `oneof`_ ``_advertiser_upload_date_time``.
        valid_transaction_fraction (float):
            The fraction of transactions that are valid.
            Invalid transactions may include invalid formats
            or values. Required.
            The fraction needs to be between 0 and 1
            (excluding 0).

            This field is a member of `oneof`_ ``_valid_transaction_fraction``.
        partner_match_fraction (float):
            The fraction of valid transactions that are
            matched to a third party assigned user ID on the
            partner side. Required.
            The fraction needs to be between 0 and 1
            (excluding 0).

            This field is a member of `oneof`_ ``_partner_match_fraction``.
        partner_upload_fraction (float):
            The fraction of valid transactions that are
            uploaded by the partner to Google.
            Required.
            The fraction needs to be between 0 and 1
            (excluding 0).

            This field is a member of `oneof`_ ``_partner_upload_fraction``.
        bridge_map_version_id (str):
            Version of partner IDs to be used for
            uploads. Required.

            This field is a member of `oneof`_ ``_bridge_map_version_id``.
        partner_id (int):
            ID of the third party partner updating the
            transaction feed.

            This field is a member of `oneof`_ ``_partner_id``.
    """

    advertiser_upload_date_time: str = proto.Field(
        proto.STRING,
        number=7,
        optional=True,
    )
    valid_transaction_fraction: float = proto.Field(
        proto.DOUBLE,
        number=8,
        optional=True,
    )
    partner_match_fraction: float = proto.Field(
        proto.DOUBLE,
        number=9,
        optional=True,
    )
    partner_upload_fraction: float = proto.Field(
        proto.DOUBLE,
        number=10,
        optional=True,
    )
    bridge_map_version_id: str = proto.Field(
        proto.STRING,
        number=11,
        optional=True,
    )
    partner_id: int = proto.Field(
        proto.INT64,
        number=12,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



#!/usr/bin/env python
# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This example shows how to create a Performance Max campaign.

For more information about Performance Max campaigns, see
https://developers.google.com/google-ads/api/docs/performance-max/overview

Prerequisites:
- You must have at least one conversion action in the account. For
more about conversion actions, see
https://developers.google.com/google-ads/api/docs/conversions/overview#conversion_actions

This example uses the default customer conversion goals. For an example
of setting campaign-specific conversion goals, see
shopping_ads/add_performance_max_retail_campaign.py
"""


import argparse
from datetime import datetime, timedelta
import sys
from typing import List, Optional, Iterable
from uuid import uuid4

from examples.utils.example_helpers import get_image_bytes_from_url
from google.ads.googleads.client import GoogleAdsClient
from google.ads.googleads.errors import GoogleAdsException
from google.ads.googleads.util import convert_snake_case_to_upper_case
from google.ads.googleads.v22.enums.types.asset_field_type import (
    AssetFieldTypeEnum,
)
from google.ads.googleads.v22.resources.types.asset import Asset
from google.ads.googleads.v22.resources.types.asset_group import AssetGroup
from google.ads.googleads.v22.resources.types.asset_group_asset import (
    AssetGroupAsset,
)
from google.ads.googleads.v22.resources.types.asset_group_signal import (
    AssetGroupSignal,
)
from google.ads.googleads.v22.resources.types.campaign import Campaign
from google.ads.googleads.v22.resources.types.campaign_asset import (
    CampaignAsset,
)
from google.ads.googleads.v22.resources.types.campaign_budget import (
    CampaignBudget,
)
from google.ads.googleads.v22.resources.types.campaign_criterion import (
    CampaignCriterion,
)
from google.ads.googleads.v22.services.services.asset_group_service import (
    AssetGroupServiceClient,
)
from google.ads.googleads.v22.services.services.asset_service import (
    AssetServiceClient,
)
from google.ads.googleads.v22.services.services.campaign_service import (
    CampaignServiceClient,
)
from google.ads.googleads.v22.services.services.geo_target_constant_service import (
    GeoTargetConstantServiceClient,
)
from google.ads.googleads.v22.services.services.google_ads_service import (
    GoogleAdsServiceClient,
)
from google.ads.googleads.v22.services.types.campaign_budget_service import (
    CampaignBudgetOperation,
)
from google.ads.googleads.v22.services.types.google_ads_service import (
    MutateGoogleAdsResponse,
    MutateOperation,
    MutateOperationResponse,
)


# We specify temporary IDs that are specific to a single mutate request.
# Temporary IDs are always negative and unique within one mutate request.
#
# See https://developers.google.com/google-ads/api/docs/mutating/best-practices
# for further details.
#
# These temporary IDs are fixed because they are used in multiple places.
_BUDGET_TEMPORARY_ID = "-1"
_PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID = "-2"
_ASSET_GROUP_TEMPORARY_ID = "-3"

# There are also entities that will be created in the same request but do not
# need to be fixed temporary IDs because they are referenced only once.
next_temp_id = int(_ASSET_GROUP_TEMPORARY_ID) - 1


# [START add_performance_max_campaign]
def main(
    client: GoogleAdsClient,
    customer_id: str,
    audience_id: Optional[str],
    brand_guidelines_enabled: bool,
) -> None:
    """The main method that creates all necessary entities for the example.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        audience_id: an optional audience ID.
        brand_guidelines_enabled: a boolean value indicating if the campaign is
          enabled for brand guidelines.
    """
    # [START add_performance_max_campaign_1]
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    # Performance Max campaigns require that repeated assets such as headlines
    # and descriptions be created before the campaign.
    # For the list of required assets for a Performance Max campaign, see
    # https://developers.google.com/google-ads/api/docs/performance-max/assets
    #
    # Create the headlines.
    headline_asset_resource_names: List[str] = create_multiple_text_assets(
        client,
        customer_id,
        [
            "Travel",
            "Travel Reviews",
            "Book travel",
        ],
    )
    # Create the descriptions.
    description_asset_resource_names: List[str] = create_multiple_text_assets(
        client,
        customer_id,
        [
            "Take to the air!",
            "Fly to the sky!",
        ],
    )

    # The below methods create and return MutateOperations that we later
    # provide to the GoogleAdsService.Mutate method in order to create the
    # entities in a single request. Since the entities for a Performance Max
    # campaign are closely tied to one-another, it's considered a best practice
    # to create them in a single Mutate request so they all complete
    # successfully or fail entirely, leaving no orphaned entities. See:
    # https://developers.google.com/google-ads/api/docs/mutating/overview
    campaign_budget_operation: MutateOperation = (
        create_campaign_budget_operation(
            client,
            customer_id,
        )
    )
    performance_max_campaign_operation: MutateOperation = (
        create_performance_max_campaign_operation(
            client,
            customer_id,
            brand_guidelines_enabled,
        )
    )
    campaign_criterion_operations: List[MutateOperation] = (
        create_campaign_criterion_operations(
            client,
            customer_id,
        )
    )
    asset_group_operations: List[MutateOperation] = (
        create_asset_group_operation(
            client,
            customer_id,
            headline_asset_resource_names,
            description_asset_resource_names,
            brand_guidelines_enabled,
        )
    )
    asset_group_signal_operations: List[MutateOperation] = (
        create_asset_group_signal_operations(client, customer_id, audience_id)
    )

    mutate_operations: List[MutateOperation] = [
        # It's important to create these entities in this order because
        # they depend on each other.
        campaign_budget_operation,
        performance_max_campaign_operation,
        # Expand the list of multiple operations into the list of
        # other mutate operations
        *campaign_criterion_operations,
        *asset_group_operations,
        *asset_group_signal_operations,
    ]

    # Send the operations in a single Mutate request.
    response: MutateGoogleAdsResponse = googleads_service.mutate(
        customer_id=customer_id, mutate_operations=mutate_operations
    )

    print_response_details(response)
    # [END add_performance_max_campaign_1]


# [START add_performance_max_campaign_2]
def create_campaign_budget_operation(
    client: GoogleAdsClient,
    customer_id: str,
) -> MutateOperation:
    """Creates a MutateOperation that creates a new CampaignBudget.

    A temporary ID will be assigned to this campaign budget so that it can be
    referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        a MutateOperation that creates a CampaignBudget.
    """
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    campaign_budget_operation: CampaignBudgetOperation = (
        mutate_operation.campaign_budget_operation
    )
    campaign_budget: CampaignBudget = campaign_budget_operation.create
    campaign_budget.name = f"Performance Max campaign budget #{uuid4()}"
    # The budget period already defaults to DAILY.
    campaign_budget.amount_micros = 50000000
    campaign_budget.delivery_method = (
        client.enums.BudgetDeliveryMethodEnum.STANDARD
    )
    # A Performance Max campaign cannot use a shared campaign budget.
    campaign_budget.explicitly_shared = False

    # Set a temporary ID in the budget's resource name so it can be referenced
    # by the campaign in later steps.
    campaign_budget.resource_name = client.get_service(
        "CampaignBudgetService"
    ).campaign_budget_path(customer_id, _BUDGET_TEMPORARY_ID)

    return mutate_operation
    # [END add_performance_max_campaign_2]


# [START add_performance_max_campaign_3]
def create_performance_max_campaign_operation(
    client: GoogleAdsClient,
    customer_id: str,
    brand_guidelines_enabled: bool,
) -> MutateOperation:
    """Creates a MutateOperation that creates a new Performance Max campaign.

    A temporary ID will be assigned to this campaign so that it can
    be referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        brand_guidelines_enabled: a boolean value indicating if the campaign is
          enabled for brand guidelines.

    Returns:
        a MutateOperation that creates a campaign.
    """
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    campaign: Campaign = mutate_operation.campaign_operation.create
    campaign.name = f"Performance Max campaign #{uuid4()}"
    # Set the campaign status as PAUSED. The campaign is the only entity in
    # the mutate request that should have its status set.
    campaign.status = client.enums.CampaignStatusEnum.PAUSED
    # All Performance Max campaigns have an advertising_channel_type of
    # PERFORMANCE_MAX. The advertising_channel_sub_type should not be set.
    campaign.advertising_channel_type = (
        client.enums.AdvertisingChannelTypeEnum.PERFORMANCE_MAX
    )
    # Bidding strategy must be set directly on the campaign.
    # Setting a portfolio bidding strategy by resource name is not supported.
    # Max Conversion and Maximize Conversion Value are the only strategies
    # supported for Performance Max campaigns.
    # An optional ROAS (Return on Advertising Spend) can be set for
    # maximize_conversion_value. The ROAS value must be specified as a ratio in
    # the API. It is calculated by dividing "total value" by "total spend".
    # For more information on Maximize Conversion Value, see the support
    # article: http://support.google.com/google-ads/answer/7684216.
    # A target_roas of 3.5 corresponds to a 350% return on ad spend.
    campaign.bidding_strategy_type = (
        client.enums.BiddingStrategyTypeEnum.MAXIMIZE_CONVERSION_VALUE
    )
    campaign.maximize_conversion_value.target_roas = 3.5

    # Set if the campaign is enabled for brand guidelines. For more information
    # on brand guidelines, see https://support.google.com/google-ads/answer/14934472.
    campaign.brand_guidelines_enabled = brand_guidelines_enabled

    # Assign the resource name with a temporary ID.
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )
    campaign.resource_name = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    # Set the budget using the given budget resource name.
    campaign.campaign_budget = campaign_service.campaign_budget_path(
        customer_id, _BUDGET_TEMPORARY_ID
    )

    # Declare whether or not this campaign serves political ads targeting the
    # EU. Valid values are:
    #   CONTAINS_EU_POLITICAL_ADVERTISING
    #   DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    campaign.contains_eu_political_advertising = (
        client.enums.EuPoliticalAdvertisingStatusEnum.DOES_NOT_CONTAIN_EU_POLITICAL_ADVERTISING
    )

    # Optional fields
    campaign.start_date = (datetime.now() + timedelta(1)).strftime("%Y%m%d")
    campaign.end_date = (datetime.now() + timedelta(365)).strftime("%Y%m%d")

    # [START add_pmax_asset_automation_settings]
    # Configures the optional opt-in/out status for asset automation settings.
    for asset_automation_type_enum in [
        client.enums.AssetAutomationTypeEnum.GENERATE_IMAGE_EXTRACTION,
        client.enums.AssetAutomationTypeEnum.FINAL_URL_EXPANSION_TEXT_ASSET_AUTOMATION,
        client.enums.AssetAutomationTypeEnum.TEXT_ASSET_AUTOMATION,
        client.enums.AssetAutomationTypeEnum.GENERATE_ENHANCED_YOUTUBE_VIDEOS,
        client.enums.AssetAutomationTypeEnum.GENERATE_IMAGE_ENHANCEMENT
    ]:
        asset_automattion_setting: Campaign.AssetAutomationSetting = client.get_type("Campaign").AssetAutomationSetting()
        asset_automattion_setting.asset_automation_type = asset_automation_type_enum
        asset_automattion_setting.asset_automation_status = client.enums.AssetAutomationStatusEnum.OPTED_IN
        campaign.asset_automation_settings.append(asset_automattion_setting)
        # [END add_pmax_asset_automation_settings]

    return mutate_operation
    # [END add_performance_max_campaign_3]


# [START add_performance_max_campaign_4]
def create_campaign_criterion_operations(
    client: GoogleAdsClient,
    customer_id: str,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create new campaign criteria.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.

    Returns:
        a list of MutateOperations that create new campaign criteria.
    """
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )
    geo_target_constant_service: GeoTargetConstantServiceClient = (
        client.get_service("GeoTargetConstantService")
    )
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    operations: List[MutateOperation] = []
    # Set the LOCATION campaign criteria.
    # Target all of New York City except Brooklyn.
    # Location IDs are listed here:
    # https://developers.google.com/google-ads/api/reference/data/geotargets
    # and they can also be retrieved using the GeoTargetConstantService as shown
    # here: https://developers.google.com/google-ads/api/docs/targeting/location-targeting
    #
    # We will add one positive location target for New York City (ID=1023191)
    # and one negative location target for Brooklyn (ID=1022762).
    # First, add the positive (negative = False) for New York City.
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    campaign_criterion: CampaignCriterion = (
        mutate_operation.campaign_criterion_operation.create
    )
    campaign_criterion.campaign = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    campaign_criterion.location.geo_target_constant = (
        geo_target_constant_service.geo_target_constant_path("1023191")
    )
    campaign_criterion.negative = False
    operations.append(mutate_operation)

    # Next add the negative target for Brooklyn.
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    campaign_criterion: CampaignCriterion = (
        mutate_operation.campaign_criterion_operation.create
    )
    campaign_criterion.campaign = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    campaign_criterion.location.geo_target_constant = (
        geo_target_constant_service.geo_target_constant_path("1022762")
    )
    campaign_criterion.negative = True
    operations.append(mutate_operation)

    # Set the LANGUAGE campaign criterion.
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    campaign_criterion: CampaignCriterion = (
        mutate_operation.campaign_criterion_operation.create
    )
    campaign_criterion.campaign = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    # Set the language.
    # For a list of all language codes, see:
    # https://developers.google.com/google-ads/api/reference/data/codes-formats#expandable-7
    campaign_criterion.language.language_constant = (
        googleads_service.language_constant_path("1000")
    )  # English
    operations.append(mutate_operation)

    return operations
    # [END add_performance_max_campaign_4]


# [START add_performance_max_campaign_5]
def create_multiple_text_assets(
    client: GoogleAdsClient, customer_id: str, texts: List[str]
) -> List[str]:
    """Creates multiple text assets and returns the list of resource names.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        texts: a list of strings, each of which will be used to create a text
          asset.

    Returns:
        asset_resource_names: a list of asset resource names.
    """
    # Here again we use the GoogleAdService to create multiple text
    # assets in a single request.
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )

    operations: List[MutateOperation] = []
    for text in texts:
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        asset: Asset = mutate_operation.asset_operation.create
        asset.text_asset.text = text
        operations.append(mutate_operation)

    # Send the operations in a single Mutate request.
    response: MutateGoogleAdsResponse = googleads_service.mutate(
        customer_id=customer_id,
        mutate_operations=operations,
    )
    asset_resource_names: List[str] = []
    for result in response.mutate_operation_responses:
        if result._pb.HasField("asset_result"):
            asset_resource_names.append(result.asset_result.resource_name)
    print_response_details(response)
    return asset_resource_names
    # [END add_performance_max_campaign_5]


# [START add_performance_max_campaign_6]
def create_asset_group_operation(
    client: GoogleAdsClient,
    customer_id: str,
    headline_asset_resource_names: List[str],
    description_asset_resource_names: List[str],
    brand_guidelines_enabled: bool,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create a new asset_group.

    A temporary ID will be assigned to this asset group so that it can
    be referenced by other objects being created in the same Mutate request.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        headline_asset_resource_names: a list of headline resource names.
        description_asset_resource_names: a list of description resource names.
        brand_guidelines_enabled: a boolean value indicating if the campaign is
          enabled for brand guidelines.

    Returns:
        MutateOperations that create a new asset group and related assets.
    """
    asset_group_service: AssetGroupServiceClient = client.get_service(
        "AssetGroupService"
    )
    campaign_service: CampaignServiceClient = client.get_service(
        "CampaignService"
    )

    operations: List[MutateOperation] = []

    # Create the AssetGroup
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    asset_group: AssetGroup = mutate_operation.asset_group_operation.create
    asset_group.name = f"Performance Max asset group #{uuid4()}"
    asset_group.campaign = campaign_service.campaign_path(
        customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
    )
    asset_group.final_urls.append("http://www.example.com")
    asset_group.final_mobile_urls.append("http://www.example.com")
    asset_group.status = client.enums.AssetGroupStatusEnum.PAUSED
    asset_group.resource_name = asset_group_service.asset_group_path(
        customer_id,
        _ASSET_GROUP_TEMPORARY_ID,
    )
    operations.append(mutate_operation)

    # For the list of required assets for a Performance Max campaign, see
    # https://developers.google.com/google-ads/api/docs/performance-max/assets

    # An AssetGroup is linked to an Asset by creating a new AssetGroupAsset
    # and providing:
    #   the resource name of the AssetGroup
    #   the resource name of the Asset
    #   the field_type of the Asset in this AssetGroup.
    #
    # To learn more about AssetGroups, see
    # https://developers.google.com/google-ads/api/docs/performance-max/asset-groups

    # Link the previously created multiple text assets.

    # Link the headline assets.
    for resource_name in headline_asset_resource_names:
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        asset_group_asset: AssetGroupAsset = (
            mutate_operation.asset_group_asset_operation.create
        )
        asset_group_asset.field_type = client.enums.AssetFieldTypeEnum.HEADLINE
        asset_group_asset.asset_group = asset_group_service.asset_group_path(
            customer_id,
            _ASSET_GROUP_TEMPORARY_ID,
        )
        asset_group_asset.asset = resource_name
        operations.append(mutate_operation)

    #  Link the description assets.
    for resource_name in description_asset_resource_names:
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        asset_group_asset: AssetGroupAsset = (
            mutate_operation.asset_group_asset_operation.create
        )
        asset_group_asset.field_type = (
            client.enums.AssetFieldTypeEnum.DESCRIPTION
        )
        asset_group_asset.asset_group = asset_group_service.asset_group_path(
            customer_id,
            _ASSET_GROUP_TEMPORARY_ID,
        )
        asset_group_asset.asset = resource_name
        operations.append(mutate_operation)

    # Create and link the long headline text asset.
    mutate_operations: List[MutateOperation] = create_and_link_text_asset(
        client,
        customer_id,
        "Travel the World",
        client.enums.AssetFieldTypeEnum.LONG_HEADLINE,
    )
    operations.extend(mutate_operations)

    # Create and link the business name and logo asset.
    mutate_operations: List[MutateOperation] = create_and_link_brand_assets(
        client,
        customer_id,
        brand_guidelines_enabled,
        "Interplanetary Cruises",
        "https://gaagl.page.link/bjYi",
        "Marketing Logo",
    )
    operations.extend(mutate_operations)

    # Create and link the image assets.

    # Create and link the Marketing Image Asset.
    mutate_operations: List[MutateOperation] = create_and_link_image_asset(
        client,
        customer_id,
        "https://gaagl.page.link/Eit5",
        client.enums.AssetFieldTypeEnum.MARKETING_IMAGE,
        "Marketing Image",
    )
    operations.extend(mutate_operations)

    # Create and link the Square Marketing Image Asset.
    mutate_operations: List[MutateOperation] = create_and_link_image_asset(
        client,
        customer_id,
        "https://gaagl.page.link/bjYi",
        client.enums.AssetFieldTypeEnum.SQUARE_MARKETING_IMAGE,
        "Square Marketing Image",
    )
    operations.extend(mutate_operations)
    return operations
    # [END add_performance_max_campaign_6]


# [START add_performance_max_campaign_7]
def create_and_link_text_asset(
    client: GoogleAdsClient,
    customer_id: str,
    text: str,
    field_type: AssetFieldTypeEnum.AssetFieldType,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create a new linked text asset.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        text: the text of the asset to be created.
        field_type: the field_type of the new asset in the AssetGroupAsset.

    Returns:
        MutateOperations that create a new linked text asset.
    """
    global next_temp_id
    operations: List[MutateOperation] = []
    asset_service: AssetServiceClient = client.get_service("AssetService")
    asset_group_service: AssetGroupServiceClient = client.get_service(
        "AssetGroupService"
    )

    # Create the Text Asset.
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    asset: Asset = mutate_operation.asset_operation.create
    asset.resource_name = asset_service.asset_path(customer_id, next_temp_id)
    asset.text_asset.text = text
    operations.append(mutate_operation)

    # Create an AssetGroupAsset to link the Asset to the AssetGroup.
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    asset_group_asset: AssetGroupAsset = (
        mutate_operation.asset_group_asset_operation.create
    )
    asset_group_asset.field_type = field_type
    asset_group_asset.asset_group = asset_group_service.asset_group_path(
        customer_id,
        _ASSET_GROUP_TEMPORARY_ID,
    )
    asset_group_asset.asset = asset_service.asset_path(
        customer_id, next_temp_id
    )
    operations.append(mutate_operation)

    next_temp_id -= 1
    return operations
    # [END add_performance_max_campaign_7]


# [START add_performance_max_campaign_8]
def create_and_link_image_asset(
    client: GoogleAdsClient,
    customer_id: str,
    url: str,
    field_type: AssetFieldTypeEnum.AssetFieldType,
    asset_name: str,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create a new linked image asset.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        url: the url of the image to be retrieved and put into an asset.
        field_type: the field_type of the new asset in the AssetGroupAsset.
        asset_name: the asset name.

    Returns:
        MutateOperations that create a new linked image asset.
    """
    global next_temp_id
    operations: List[MutateOperation] = []
    asset_service: AssetServiceClient = client.get_service("AssetService")
    asset_group_service: AssetGroupServiceClient = client.get_service(
        "AssetGroupService"
    )

    # Create the Image Asset.
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    asset: Asset = mutate_operation.asset_operation.create
    asset.resource_name = asset_service.asset_path(customer_id, next_temp_id)
    # Provide a unique friendly name to identify your asset.
    # When there is an existing image asset with the same content but a different
    # name, the new name will be dropped silently.
    asset.name = asset_name
    asset.type_ = client.enums.AssetTypeEnum.IMAGE
    asset.image_asset.data = get_image_bytes_from_url(url)
    operations.append(mutate_operation)

    # Create an AssetGroupAsset to link the Asset to the AssetGroup.
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    asset_group_asset: AssetGroupAsset = (
        mutate_operation.asset_group_asset_operation.create
    )
    asset_group_asset.field_type = field_type
    asset_group_asset.asset_group = asset_group_service.asset_group_path(
        customer_id,
        _ASSET_GROUP_TEMPORARY_ID,
    )
    asset_group_asset.asset = asset_service.asset_path(
        customer_id, next_temp_id
    )
    operations.append(mutate_operation)

    next_temp_id -= 1
    return operations
    # [END add_performance_max_campaign_8]

# [START create_and_link_brand_assets]
def create_and_link_brand_assets(
    client: GoogleAdsClient,
    customer_id: str,
    brand_guidelines_enabled: bool,
    business_name: str,
    logo_url: str,
    logo_name: str,
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that create linked brand assets.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        brand_guidelines_enabled: a boolean value indicating if the campaign is
          enabled for brand guidelines.
        business_name: the business name text to be put into an asset.
        logo_url: the url of the logo to be retrieved and put into an asset.
        logo_name: the asset name of the logo.

    Returns:
        MutateOperations that create linked brand assets.
    """
    global next_temp_id
    operations: List[MutateOperation] = []
    asset_service: AssetServiceClient = client.get_service("AssetService")

    # Create the Text Asset.
    text_asset_temp_id = next_temp_id
    next_temp_id -= 1

    text_mutate_operation = client.get_type("MutateOperation")
    text_asset: Asset = text_mutate_operation.asset_operation.create
    text_asset.resource_name = asset_service.asset_path(
        customer_id, text_asset_temp_id
    )
    text_asset.text_asset.text = business_name
    operations.append(text_mutate_operation)

    # Create the Image Asset.
    image_asset_temp_id = next_temp_id
    next_temp_id -= 1

    image_mutate_operation = client.get_type("MutateOperation")
    image_asset: Asset = image_mutate_operation.asset_operation.create
    image_asset.resource_name = asset_service.asset_path(
        customer_id, image_asset_temp_id
    )
    # Provide a unique friendly name to identify your asset.
    # When there is an existing image asset with the same content but a different
    # name, the new name will be dropped silently.
    image_asset.name = logo_name
    image_asset.type_ = client.enums.AssetTypeEnum.IMAGE
    image_asset.image_asset.data = get_image_bytes_from_url(logo_url)
    operations.append(image_mutate_operation)

    if brand_guidelines_enabled:
        # Create CampaignAsset resources to link the Asset resources to the Campaign.
        campaign_service: CampaignServiceClient = client.get_service(
            "CampaignService"
        )

        business_name_mutate_operation: MutateOperation = client.get_type(
            "MutateOperation"
        )
        business_name_campaign_asset: CampaignAsset = (
            business_name_mutate_operation.campaign_asset_operation.create
        )
        business_name_campaign_asset.field_type = (
            client.enums.AssetFieldTypeEnum.BUSINESS_NAME
        )
        business_name_campaign_asset.campaign = campaign_service.campaign_path(
            customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
        )
        business_name_campaign_asset.asset = asset_service.asset_path(
            customer_id, text_asset_temp_id
        )
        operations.append(business_name_mutate_operation)

        logo_mutate_operation: MutateOperation = client.get_type(
            "MutateOperation"
        )
        logo_campaign_asset: CampaignAsset = (
            logo_mutate_operation.campaign_asset_operation.create
        )
        logo_campaign_asset.field_type = client.enums.AssetFieldTypeEnum.LOGO
        logo_campaign_asset.campaign = campaign_service.campaign_path(
            customer_id, _PERFORMANCE_MAX_CAMPAIGN_TEMPORARY_ID
        )
        logo_campaign_asset.asset = asset_service.asset_path(
            customer_id, image_asset_temp_id
        )
        operations.append(logo_mutate_operation)

    else:
        # Create AssetGroupAsset resources to link the Asset resources to the AssetGroup.
        asset_group_service: AssetGroupServiceClient = client.get_service(
            "AssetGroupService"
        )

        business_name_mutate_operation: MutateOperation = client.get_type(
            "MutateOperation"
        )
        business_name_asset_group_asset: AssetGroupAsset = (
            business_name_mutate_operation.asset_group_asset_operation.create
        )
        business_name_asset_group_asset.field_type = (
            client.enums.AssetFieldTypeEnum.BUSINESS_NAME
        )
        business_name_asset_group_asset.asset_group = (
            asset_group_service.asset_group_path(
                customer_id,
                _ASSET_GROUP_TEMPORARY_ID,
            )
        )
        business_name_asset_group_asset.asset = asset_service.asset_path(
            customer_id, text_asset_temp_id
        )
        operations.append(business_name_mutate_operation)

        logo_mutate_operation: MutateOperation = client.get_type(
            "MutateOperation"
        )
        logo_asset_group_asset: AssetGroupAsset = (
            logo_mutate_operation.asset_group_asset_operation.create
        )
        logo_asset_group_asset.field_type = client.enums.AssetFieldTypeEnum.LOGO
        logo_asset_group_asset.asset_group = (
            asset_group_service.asset_group_path(
                customer_id,
                _ASSET_GROUP_TEMPORARY_ID,
            )
        )
        logo_asset_group_asset.asset = asset_service.asset_path(
            customer_id, image_asset_temp_id
        )
        operations.append(logo_mutate_operation)

    return operations
    # [END create_and_link_brand_assets]


def print_response_details(response: MutateGoogleAdsResponse) -> None:
    """Prints the details of a MutateGoogleAdsResponse.

    Parses the "response" oneof field name and uses it to extract the new
    entity's name and resource name.

    Args:
        response: a MutateGoogleAdsResponse object.
    """
    # Parse the Mutate response to print details about the entities that
    # were created by the request.
    results: Iterable[MutateOperation] = response.mutate_operation_responses
    suffix = "_result"
    for result in results:
        for field_descriptor, value in result._pb.ListFields():
            if field_descriptor.name.endswith(suffix):
                name = field_descriptor.name[: -len(suffix)]
            else:
                name = field_descriptor.name
            print(
                f"Created a(n) {convert_snake_case_to_upper_case(name)} with "
                f"{str(value).strip()}."
            )


def create_asset_group_signal_operations(
    client: GoogleAdsClient, customer_id: str, audience_id: Optional[str]
) -> List[MutateOperation]:
    """Creates a list of MutateOperations that may create asset group signals.

    Args:
        client: an initialized GoogleAdsClient instance.
        customer_id: a client customer ID.
        audience_id: an optional audience ID.

    Returns:
        MutateOperations that create new asset group signals.
    """
    googleads_service: GoogleAdsServiceClient = client.get_service(
        "GoogleAdsService"
    )
    asset_group_resource_name: str = googleads_service.asset_group_path(
        customer_id, _ASSET_GROUP_TEMPORARY_ID
    )

    operations: List[MutateOperation] = []

    if audience_id:
        # Create an audience asset group signal.
        # To learn more about Audience Signals, see:
        # https://developers.google.com/google-ads/api/performance-max/asset-group-signals#audiences
        # [START add_performance_max_campaign_9]
        mutate_operation: MutateOperation = client.get_type("MutateOperation")
        operation: AssetGroupSignal = (
            mutate_operation.asset_group_signal_operation.create
        )
        operation.asset_group = asset_group_resource_name
        operation.audience.audience = googleads_service.audience_path(
            customer_id, audience_id
        )
        operations.append(mutate_operation)
        # [END add_performance_max_campaign_9]

    # Create a search theme asset group signal.
    # To learn more about Search Themes Signals, see:
    # https://developers.google.com/google-ads/api/performance-max/asset-group-signals#search_themes
    # [START add_performance_max_campaign_10]
    mutate_operation: MutateOperation = client.get_type("MutateOperation")
    operation: AssetGroupSignal = (
        mutate_operation.asset_group_signal_operation.create
    )
    operation.asset_group = asset_group_resource_name
    operation.search_theme.text = "travel"
    operations.append(mutate_operation)
    # [END add_performance_max_campaign_10]

    return operations


# [END add_performance_max_campaign]

if __name__ == "__main__":
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=("Creates a Performance Max campaign.")
    )
    # The following argument(s) should be provided to run the example.
    parser.add_argument(
        "-c",
        "--customer_id",
        type=str,
        required=True,
        help="The Google Ads customer ID.",
    )
    parser.add_argument(
        "-a",
        "--audience_id",
        type=str,
        help="The ID of an audience.",
    )
    parser.add_argument(
        "-b",
        "--brand_guidelines_enabled",
        type=bool,
        default=False,
        help=(
            "A boolean value indicating if the created campaign is enabled "
            "for brand guidelines."
        ),
    )

    args: argparse.Namespace = parser.parse_args()

    # GoogleAdsClient will read the google-ads.yaml configuration file in the
    # home directory if none is specified.
    googleads_client: GoogleAdsClient = GoogleAdsClient.load_from_storage(
        version="v22"
    )

    try:
        main(
            googleads_client,
            args.customer_id,
            args.audience_id,
            args.brand_guidelines_enabled,
        )
    except GoogleAdsException as ex:
        print(
            f'Request with ID "{ex.request_id}" failed with status '
            f'"{ex.error.code().name}" and includes the following errors:'
        )
        for error in ex.failure.errors:
            print(f'Error with message "{error.message}".')
            if error.location:
                for field_path_element in error.location.field_path_elements:
                    print(f"\t\tOn field: {field_path_element.field_name}")
        sys.exit(1)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v21.resources.types import ad_group_ad_label
from google.rpc import status_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.services",
    marshal="google.ads.googleads.v21",
    manifest={
        "MutateAdGroupAdLabelsRequest",
        "AdGroupAdLabelOperation",
        "MutateAdGroupAdLabelsResponse",
        "MutateAdGroupAdLabelResult",
    },
)


class MutateAdGroupAdLabelsRequest(proto.Message):
    r"""Request message for
    [AdGroupAdLabelService.MutateAdGroupAdLabels][google.ads.googleads.v21.services.AdGroupAdLabelService.MutateAdGroupAdLabels].

    Attributes:
        customer_id (str):
            Required. ID of the customer whose ad group
            ad labels are being modified.
        operations (MutableSequence[google.ads.googleads.v21.services.types.AdGroupAdLabelOperation]):
            Required. The list of operations to perform
            on ad group ad labels.
        partial_failure (bool):
            If true, successful operations will be
            carried out and invalid operations will return
            errors. If false, all operations will be carried
            out in one transaction if and only if they are
            all valid. Default is false.
        validate_only (bool):
            If true, the request is validated but not
            executed. Only errors are returned, not results.
    """

    customer_id: str = proto.Field(
        proto.STRING,
        number=1,
    )
    operations: MutableSequence["AdGroupAdLabelOperation"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="AdGroupAdLabelOperation",
        )
    )
    partial_failure: bool = proto.Field(
        proto.BOOL,
        number=3,
    )
    validate_only: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class AdGroupAdLabelOperation(proto.Message):
    r"""A single operation (create, remove) on an ad group ad label.

    This message has `oneof`_ fields (mutually exclusive fields).
    For each oneof, at most one member field can be set at the same time.
    Setting any member of the oneof automatically clears all other
    members.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        create (google.ads.googleads.v21.resources.types.AdGroupAdLabel):
            Create operation: No resource name is
            expected for the new ad group ad label.

            This field is a member of `oneof`_ ``operation``.
        remove (str):
            Remove operation: A resource name for the ad group ad label
            being removed, in this format:

            ``customers/{customer_id}/adGroupAdLabels/{ad_group_id}~{ad_id}~{label_id}``

            This field is a member of `oneof`_ ``operation``.
    """

    create: ad_group_ad_label.AdGroupAdLabel = proto.Field(
        proto.MESSAGE,
        number=1,
        oneof="operation",
        message=ad_group_ad_label.AdGroupAdLabel,
    )
    remove: str = proto.Field(
        proto.STRING,
        number=2,
        oneof="operation",
    )


class MutateAdGroupAdLabelsResponse(proto.Message):
    r"""Response message for an ad group ad labels mutate.

    Attributes:
        partial_failure_error (google.rpc.status_pb2.Status):
            Errors that pertain to operation failures in the partial
            failure mode. Returned only when partial_failure = true and
            all errors occur inside the operations. If any errors occur
            outside the operations (for example, auth errors), we return
            an RPC level error.
        results (MutableSequence[google.ads.googleads.v21.services.types.MutateAdGroupAdLabelResult]):
            All results for the mutate.
    """

    partial_failure_error: status_pb2.Status = proto.Field(
        proto.MESSAGE,
        number=3,
        message=status_pb2.Status,
    )
    results: MutableSequence["MutateAdGroupAdLabelResult"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message="MutateAdGroupAdLabelResult",
        )
    )


class MutateAdGroupAdLabelResult(proto.Message):
    r"""The result for an ad group ad label mutate.

    Attributes:
        resource_name (str):
            Returned for successful operations.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v21.enums.types import customizer_attribute_type


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.common",
    marshal="google.ads.googleads.v21",
    manifest={
        "CustomizerValue",
    },
)


class CustomizerValue(proto.Message):
    r"""A customizer value that is referenced in customizer linkage
    entities like CustomerCustomizer, CampaignCustomizer, etc.

    Attributes:
        type_ (google.ads.googleads.v21.enums.types.CustomizerAttributeTypeEnum.CustomizerAttributeType):
            Required. The data type for the customizer value. It must
            match the attribute type. The string_value content must
            match the constraints associated with the type.
        string_value (str):
            Required. Value to insert in creative text.
            Customizer values of all types are stored as
            string to make formatting unambiguous.
    """

    type_: (
        customizer_attribute_type.CustomizerAttributeTypeEnum.CustomizerAttributeType
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=customizer_attribute_type.CustomizerAttributeTypeEnum.CustomizerAttributeType,
    )
    string_value: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableSequence

import proto  # type: ignore

from google.ads.googleads.v22.enums.types import ad_strength as gage_ad_strength
from google.ads.googleads.v22.enums.types import ad_strength_action_item_type
from google.ads.googleads.v22.enums.types import (
    asset_coverage_video_aspect_ratio_requirement,
)
from google.ads.googleads.v22.enums.types import (
    asset_field_type as gage_asset_field_type,
)
from google.ads.googleads.v22.enums.types import asset_group_primary_status
from google.ads.googleads.v22.enums.types import (
    asset_group_primary_status_reason,
)
from google.ads.googleads.v22.enums.types import asset_group_status


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.resources",
    marshal="google.ads.googleads.v22",
    manifest={
        "AssetGroup",
        "AssetCoverage",
        "AdStrengthActionItem",
    },
)


class AssetGroup(proto.Message):
    r"""An asset group.
    AssetGroupAsset is used to link an asset to the asset group.
    AssetGroupSignal is used to associate a signal to an asset
    group.

    Attributes:
        resource_name (str):
            Immutable. The resource name of the asset group. Asset group
            resource names have the form:

            ``customers/{customer_id}/assetGroups/{asset_group_id}``
        id (int):
            Output only. The ID of the asset group.
        campaign (str):
            Immutable. The campaign with which this asset
            group is associated. The asset which is linked
            to the asset group.
        name (str):
            Required. Name of the asset group. Required.
            It must have a minimum length of 1 and maximum
            length of 128. It must be unique under a
            campaign.
        final_urls (MutableSequence[str]):
            A list of final URLs after all cross domain
            redirects. In performance max, by default, the
            urls are eligible for expansion unless opted
            out.
        final_mobile_urls (MutableSequence[str]):
            A list of final mobile URLs after all cross
            domain redirects. In performance max, by
            default, the urls are eligible for expansion
            unless opted out.
        status (google.ads.googleads.v22.enums.types.AssetGroupStatusEnum.AssetGroupStatus):
            The status of the asset group.
        primary_status (google.ads.googleads.v22.enums.types.AssetGroupPrimaryStatusEnum.AssetGroupPrimaryStatus):
            Output only. The primary status of the asset
            group. Provides insights into why an asset group
            is not serving or not serving optimally.
        primary_status_reasons (MutableSequence[google.ads.googleads.v22.enums.types.AssetGroupPrimaryStatusReasonEnum.AssetGroupPrimaryStatusReason]):
            Output only. Provides reasons into why an
            asset group is not serving or not serving
            optimally. It will be empty when the asset group
            is serving without issues.
        path1 (str):
            First part of text that may appear appended
            to the url displayed in the ad.
        path2 (str):
            Second part of text that may appear appended
            to the url displayed in the ad. This field can
            only be set when path1 is set.
        ad_strength (google.ads.googleads.v22.enums.types.AdStrengthEnum.AdStrength):
            Output only. Overall ad strength of this
            asset group.
        asset_coverage (google.ads.googleads.v22.resources.types.AssetCoverage):
            Output only. The asset coverage of this asset
            group.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    id: int = proto.Field(
        proto.INT64,
        number=9,
    )
    campaign: str = proto.Field(
        proto.STRING,
        number=2,
    )
    name: str = proto.Field(
        proto.STRING,
        number=3,
    )
    final_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=4,
    )
    final_mobile_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=5,
    )
    status: asset_group_status.AssetGroupStatusEnum.AssetGroupStatus = (
        proto.Field(
            proto.ENUM,
            number=6,
            enum=asset_group_status.AssetGroupStatusEnum.AssetGroupStatus,
        )
    )
    primary_status: (
        asset_group_primary_status.AssetGroupPrimaryStatusEnum.AssetGroupPrimaryStatus
    ) = proto.Field(
        proto.ENUM,
        number=11,
        enum=asset_group_primary_status.AssetGroupPrimaryStatusEnum.AssetGroupPrimaryStatus,
    )
    primary_status_reasons: MutableSequence[
        asset_group_primary_status_reason.AssetGroupPrimaryStatusReasonEnum.AssetGroupPrimaryStatusReason
    ] = proto.RepeatedField(
        proto.ENUM,
        number=12,
        enum=asset_group_primary_status_reason.AssetGroupPrimaryStatusReasonEnum.AssetGroupPrimaryStatusReason,
    )
    path1: str = proto.Field(
        proto.STRING,
        number=7,
    )
    path2: str = proto.Field(
        proto.STRING,
        number=8,
    )
    ad_strength: gage_ad_strength.AdStrengthEnum.AdStrength = proto.Field(
        proto.ENUM,
        number=10,
        enum=gage_ad_strength.AdStrengthEnum.AdStrength,
    )
    asset_coverage: "AssetCoverage" = proto.Field(
        proto.MESSAGE,
        number=13,
        message="AssetCoverage",
    )


class AssetCoverage(proto.Message):
    r"""Information about the asset coverage of an asset group.

    Attributes:
        ad_strength_action_items (MutableSequence[google.ads.googleads.v22.resources.types.AdStrengthActionItem]):
            Output only. A list of action items to
            improve the ad strength of an asset group.
    """

    ad_strength_action_items: MutableSequence["AdStrengthActionItem"] = (
        proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message="AdStrengthActionItem",
        )
    )


class AdStrengthActionItem(proto.Message):
    r"""An action item to improve the ad strength of an asset group.

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        action_item_type (google.ads.googleads.v22.enums.types.AdStrengthActionItemTypeEnum.AdStrengthActionItemType):
            Output only. The action item type.
        add_asset_details (google.ads.googleads.v22.resources.types.AdStrengthActionItem.AddAssetDetails):
            Output only. The action item details for action item type
            ADD_ASSET.

            This field is a member of `oneof`_ ``action_details``.
    """

    class AddAssetDetails(proto.Message):
        r"""The details of the asset to add.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            asset_field_type (google.ads.googleads.v22.enums.types.AssetFieldTypeEnum.AssetFieldType):
                Output only. The asset field type of the
                asset(s) to add.
            asset_count (int):
                Output only. The number of assets to add.

                This field is a member of `oneof`_ ``_asset_count``.
            video_aspect_ratio_requirement (google.ads.googleads.v22.enums.types.AssetCoverageVideoAspectRatioRequirementEnum.AssetCoverageVideoAspectRatioRequirement):
                Output only. For video field types, the required aspect
                ratio of the video. When unset and asset_field_type is
                YOUTUBE_VIDEO, the system recommends the advertiser upload
                any YouTube video, regardless of aspect ratio.

                This field is a member of `oneof`_ ``_video_aspect_ratio_requirement``.
        """

        asset_field_type: (
            gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType
        ) = proto.Field(
            proto.ENUM,
            number=1,
            enum=gage_asset_field_type.AssetFieldTypeEnum.AssetFieldType,
        )
        asset_count: int = proto.Field(
            proto.INT32,
            number=2,
            optional=True,
        )
        video_aspect_ratio_requirement: (
            asset_coverage_video_aspect_ratio_requirement.AssetCoverageVideoAspectRatioRequirementEnum.AssetCoverageVideoAspectRatioRequirement
        ) = proto.Field(
            proto.ENUM,
            number=3,
            optional=True,
            enum=asset_coverage_video_aspect_ratio_requirement.AssetCoverageVideoAspectRatioRequirementEnum.AssetCoverageVideoAspectRatioRequirement,
        )

    action_item_type: (
        ad_strength_action_item_type.AdStrengthActionItemTypeEnum.AdStrengthActionItemType
    ) = proto.Field(
        proto.ENUM,
        number=1,
        enum=ad_strength_action_item_type.AdStrengthActionItemTypeEnum.AdStrengthActionItemType,
    )
    add_asset_details: AddAssetDetails = proto.Field(
        proto.MESSAGE,
        number=2,
        oneof="action_details",
        message=AddAssetDetails,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v21.errors",
    marshal="google.ads.googleads.v21",
    manifest={
        "CampaignDraftErrorEnum",
    },
)


class CampaignDraftErrorEnum(proto.Message):
    r"""Container for enum describing possible campaign draft errors."""

    class CampaignDraftError(proto.Enum):
        r"""Enum describing possible campaign draft errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            DUPLICATE_DRAFT_NAME (2):
                A draft with this name already exists for
                this campaign.
            INVALID_STATUS_TRANSITION_FROM_REMOVED (3):
                The draft is removed and cannot be
                transitioned to another status.
            INVALID_STATUS_TRANSITION_FROM_PROMOTED (4):
                The draft has been promoted and cannot be
                transitioned to the specified status.
            INVALID_STATUS_TRANSITION_FROM_PROMOTE_FAILED (5):
                The draft has failed to be promoted and
                cannot be transitioned to the specified status.
            CUSTOMER_CANNOT_CREATE_DRAFT (6):
                This customer is not allowed to create
                drafts.
            CAMPAIGN_CANNOT_CREATE_DRAFT (7):
                This campaign is not allowed to create
                drafts.
            INVALID_DRAFT_CHANGE (8):
                This modification cannot be made on a draft.
            INVALID_STATUS_TRANSITION (9):
                The draft cannot be transitioned to the
                specified status from its current status.
            MAX_NUMBER_OF_DRAFTS_PER_CAMPAIGN_REACHED (10):
                The campaign has reached the maximum number
                of drafts that can be created for a campaign
                throughout its lifetime. No additional drafts
                can be created for this campaign. Removed drafts
                also count towards this limit.
            LIST_ERRORS_FOR_PROMOTED_DRAFT_ONLY (11):
                ListAsyncErrors was called without first
                promoting the draft.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        DUPLICATE_DRAFT_NAME = 2
        INVALID_STATUS_TRANSITION_FROM_REMOVED = 3
        INVALID_STATUS_TRANSITION_FROM_PROMOTED = 4
        INVALID_STATUS_TRANSITION_FROM_PROMOTE_FAILED = 5
        CUSTOMER_CANNOT_CREATE_DRAFT = 6
        CAMPAIGN_CANNOT_CREATE_DRAFT = 7
        INVALID_DRAFT_CHANGE = 8
        INVALID_STATUS_TRANSITION = 9
        MAX_NUMBER_OF_DRAFTS_PER_CAMPAIGN_REACHED = 10
        LIST_ERRORS_FOR_PROMOTED_DRAFT_ONLY = 11


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore

from google.ads.googleads.v20.enums.types import (
    distance_bucket as gage_distance_bucket,
)


__protobuf__ = proto.module(
    package="google.ads.googleads.v20.resources",
    marshal="google.ads.googleads.v20",
    manifest={
        "DistanceView",
    },
)


class DistanceView(proto.Message):
    r"""A distance view with metrics aggregated by the user's
    distance from an advertiser's location extensions. Each
    DistanceBucket includes all impressions that fall within its
    distance and a single impression will contribute to the metrics
    for all DistanceBuckets that include the user's distance.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        resource_name (str):
            Output only. The resource name of the distance view.
            Distance view resource names have the form:

            ``customers/{customer_id}/distanceViews/1~{distance_bucket}``
        distance_bucket (google.ads.googleads.v20.enums.types.DistanceBucketEnum.DistanceBucket):
            Output only. Grouping of user distance from
            location extensions.
        metric_system (bool):
            Output only. True if the DistanceBucket is
            using the metric system, false otherwise.

            This field is a member of `oneof`_ ``_metric_system``.
    """

    resource_name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    distance_bucket: gage_distance_bucket.DistanceBucketEnum.DistanceBucket = (
        proto.Field(
            proto.ENUM,
            number=2,
            enum=gage_distance_bucket.DistanceBucketEnum.DistanceBucket,
        )
    )
    metric_system: bool = proto.Field(
        proto.BOOL,
        number=4,
        optional=True,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations


import proto  # type: ignore


__protobuf__ = proto.module(
    package="google.ads.googleads.v22.errors",
    marshal="google.ads.googleads.v22",
    manifest={
        "QuotaErrorEnum",
    },
)


class QuotaErrorEnum(proto.Message):
    r"""Container for enum describing possible quota errors."""

    class QuotaError(proto.Enum):
        r"""Enum describing possible quota errors.

        Values:
            UNSPECIFIED (0):
                Enum unspecified.
            UNKNOWN (1):
                The received error code is not known in this
                version.
            RESOURCE_EXHAUSTED (2):
                Too many requests.
            ACCESS_PROHIBITED (3):
                Access is prohibited.
            RESOURCE_TEMPORARILY_EXHAUSTED (4):
                Too many requests in a short amount of time.
            EXCESSIVE_SHORT_TERM_QUERY_RESOURCE_CONSUMPTION (5):
                Too many expensive requests from query
                pattern over a short amount of time.
            EXCESSIVE_LONG_TERM_QUERY_RESOURCE_CONSUMPTION (6):
                Too many expensive requests from query
                pattern over an extended duration of time.
            PAYMENTS_PROFILE_ACTIVATION_RATE_LIMIT_EXCEEDED (7):
                To activate ad serving in a customer account,
                it has to be linked with a payment profile (also
                known as a Billing Customer Number, or BCN),
                which is then billed for the costs incurred by
                that customer account. This error will be thrown
                if too many customer accounts are activated in a
                short period of time for the same payment
                profile. Once this rate limit is exceeded, the
                customer should wait for a week before trying
                again, or contact Google Ads customer support to
                reset the rate limits. See
                https://support.google.com/google-ads/answer/6372658
                to learn more about this limit.
        """

        UNSPECIFIED = 0
        UNKNOWN = 1
        RESOURCE_EXHAUSTED = 2
        ACCESS_PROHIBITED = 3
        RESOURCE_TEMPORARILY_EXHAUSTED = 4
        EXCESSIVE_SHORT_TERM_QUERY_RESOURCE_CONSUMPTION = 5
        EXCESSIVE_LONG_TERM_QUERY_RESOURCE_CONSUMPTION = 6
        PAYMENTS_PROFILE_ACTIVATION_RATE_LIMIT_EXCEEDED = 7


__all__ = tuple(sorted(__protobuf__.manifest))
